#!/usr/bin/python3
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unitests for sensor.py"""
import time
import unittest

import mock
from absl.testing import absltest

from exit_speed import sensor

class TestAccelerometer(unittest.TestCase):
  """Accelerometer unittests."""

  def testSleepBasedOnHertz(self):
    now = time.time()
    cycle_time = now - 0.01
    with mock.patch.object(time, 'time') as mock_now:
      mock_now.return_value = now
      self.assertEqual(0.09,
          round(sensor.SleepBasedOnHertz(cycle_time, 10), 2))
      self.assertEqual(0.04,
          round(sensor.SleepBasedOnHertz(cycle_time, 20), 2))
      self.assertEqual(0.99,
          round(sensor.SleepBasedOnHertz(cycle_time, 1), 2))





if __name__ == '__main__':
  absltest.main()



# pytype: skip-file
#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""CSV library for converting data from Traqmate to exit speed or vice versa."""
import csv
import datetime
import time

from exit_speed import gps_pb2
from exit_speed import main


def _ReadCsvFile(filepath):
  """Reads the given CSV file."""
  reading_header = True
  start_date = None
  start_time = None
  csv_time = None
  with open(filepath) as csv_file:
    for row in csv.reader(csv_file, delimiter=','):
      if not reading_header:
        elapsed_time = row[3]
        lat = row[4]
        lon = row[5]
        alt = row[8]
        mph = row[13]
        time_str = '%s-%s' % (start_date.strip(), start_time.strip())
        csv_time = datetime.datetime.strptime(time_str, '%d/%m/%Y-%H:%M:%S.%f')
        csv_time += datetime.timedelta(seconds=float(elapsed_time.strip()))
        json_time = csv_time.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
        speed = float(mph.strip()) * 0.44704  # m/s
        lat = float(lat.strip())
        lon = float(lon.strip())
        alt = float(alt.strip())
        yield elapsed_time, json_time, lat, lon, alt, speed
      if row[0].startswith('GPS Reading'):
        reading_header = False
      if row[0].startswith('Starting Date'):
        start_date = row[1]
      if row[0].startswith('Starting Time'):
        start_time = row[1]


def ConvertTraqmateToProto(filepath):
  """Converts a Traqmate CSV file into a exit speed proto.

  Args:
    filepath: The file name and path of the Traqmate CSV file.

  Returns:
    A exit speed session proto.
  """
  es = main.ExitSpeed()
  start = time.time()
  first_elapsed = None
  for elapsed_time, json_time, lat, lon, alt, speed in _ReadCsvFile(filepath):
    point = gps_pb2.Point()
    point.lon = lon
    point.lat = lat
    point.alt = alt
    point.speed = speed
    es.ProcessReport(point)
    now = time.time()
    elapsed_time = float(elapsed_time)
    if not first_elapsed:
      first_elapsed = elapsed_time
    # Traqmate CSV files have data points every 0.025s where as our GPS sensor
    # will only record at 0.1s.
    if elapsed_time * 10 % 1 != 0:
      continue
    if start + elapsed_time > now:
      print(json_time, speed)
      sleep_duration = start + elapsed_time - first_elapsed - now
      if sleep_duration > 0:
        time.sleep(sleep_duration)
  return es.session


def ConvertProtoToTraqmate(session, filepath):
  """Converts a exit speed session to the Traqmate CSV format.

  Args:
    session: A exit speed session proto.
    filepath: The file path and name of the new CSV file.
  """
  first_point = session.laps[0].points[0]
  first_point_date = first_point.time.ToDatetime()
  last_point = session.laps[-1].points[-1]
  duration = last_point.time.ToSeconds() - first_point.time.ToSeconds()
  lap_num = 0
  with open(filepath, 'w', newline='') as csv_file:
    csv_writer = csv.writer(csv_file, delimiter=',', quoting=csv.QUOTE_MINIMAL)
    csv_writer.writerow(['Format', 'Traqmate Trackvision', 'V2'])
    csv_writer.writerow(['Track', session.track])
    csv_writer.writerow(['Vehicle', 'CORRADO'])
    csv_writer.writerow(['Driver', 'DJ'])
    csv_writer.writerow(['Starting Date',
                         first_point_date.strftime('%Y/%m/%d')])
    csv_writer.writerow(['Starting Time',
                         first_point_date.strftime('%H:%M:%03S')])
    csv_writer.writerow(['Sample Rate (samps/sec)', 10])
    csv_writer.writerow(['Duration (secs)', duration])
    csv_writer.writerow(['Elapsed Time', 'Lat (Degrees)', 'Lon (Degrees)',
                         'Altitude (meters)', 'Velocity (MPH)', 'Lap',
                         'RPM', 'TPS', 'Latitude Gs', 'Longitude Gs',
                         'Front Brake Pressure Voltage',
                         'Rear Brake Pressure Voltage'])
    for lap in session.laps:
      lap_num += 1
      for point in lap.points:
        elapsed_time = (point.time.ToNanoseconds() -
                        first_point.time.ToNanoseconds())
        speed = point.speed / 0.44704
        csv_writer.writerow([
            elapsed_time / 1000000000.0,
            point.lat,
            point.lon,
            point.alt,
            speed,
            lap_num,
            point.rpm,
            point.tps_voltage,
            point.accelerometer_x,
            point.accelerometer_y,
            point.front_brake_pressure_voltage,
            point.rear_brake_pressure_voltage
            ])



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common libaries."""
import gps

from exit_speed import gps_pb2


def PointDelta(point_a: gps_pb2.Point, point_b: gps_pb2.Point) -> float:
  """Returns the distance in meters between two points."""
  return gps.EarthDistanceSmall((point_a.lat, point_a.lon),
                                (point_b.lat, point_b.lon))



#!/usr/bin/python3
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Live data dashboard."""

from random import random

from bokeh.layouts import column
from bokeh.models import Button
from bokeh.palettes import RdYlBu3
from bokeh.plotting import figure, curdoc

# create a plot and style its properties
p = figure(x_range=(0, 100), y_range=(0, 100), toolbar_location=None)
p.border_fill_color = 'black'
p.background_fill_color = 'black'
p.outline_line_color = None
p.grid.grid_line_color = None

# add a text renderer to our plot (no data yet)
r = p.text(x=[], y=[], text=[], text_color=[], text_font_size="26px",
           text_baseline="middle", text_align="center")

i = 0

ds = r.data_source

# create a callback that will add a number in a random location
def callback():
    global i

    # BEST PRACTICE --- update .data in one step with a new dict
    new_data = dict()
    new_data['x'] = ds.data['x'] + [random()*70 + 15]
    new_data['y'] = ds.data['y'] + [random()*70 + 15]
    new_data['text_color'] = ds.data['text_color'] + [RdYlBu3[i%3]]
    new_data['text'] = ds.data['text'] + [str(i)]
    ds.data = new_data

    i = i + 1

# add a button widget and configure with the call back
button = Button(label="Press Me")
button.on_click(callback)

# put the button and plot in a layout and add to the document
curdoc().add_root(column(button, p))



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Script for replaying a data log.  Super handy for development."""
import os
import time

from absl import app
from absl import flags
from absl import logging

from exit_speed import data_logger
from exit_speed import main as exit_speed_main

FLAGS = flags.FLAGS
FLAGS.set_default('data_log_path', '/tmp')  # Dont clobber on replay.
FLAGS.set_default('led_brightness', 0.05)
flags.DEFINE_string('filepath', None, 'Path to the data file to replay')
flags.DEFINE_boolean('include_sleep', True,
                     'Adds delays to mimic real time replay of data.')


def ReplayLog(filepath, include_sleep=False):
  """Replays data, extermely useful to LED testing.

  Args:
    filepath: A string of the path of lap data.
    include_sleep: If True replays adds sleeps to simulate how data was
                   processed in real time.

  Returns:
    A exit_speed_main.ExitSpeed instance that has replayed the given data.
  """
  logging.info('Replaying %s', filepath)
  logger = data_logger.Logger(filepath)
  points = list(logger.ReadProtos())
  logging.info('Number of points %d', len(points))
  if include_sleep:
    replay_start = time.time()
    time_shift = int(replay_start * 1e9 - points[0].time.ToNanoseconds())
    session_start = None
  else:
    FLAGS.set_default('commit_cycle', 10000)
  es = exit_speed_main.ExitSpeed(live_data=not include_sleep)
  es.config['car'] = os.path.split(os.path.dirname(filepath))[1]
  for point in points:
    if include_sleep:
      point.time.FromNanoseconds(point.time.ToNanoseconds() + time_shift)
      if not session_start:
        session_start = point.time.ToMilliseconds() / 1000

    es.point = point
    es.ProcessSession()
    if include_sleep:
      run_delta = time.time() - replay_start
      point_delta = point.time.ToMilliseconds() / 1000 - session_start
      if run_delta < point_delta:
        time.sleep(point_delta - run_delta)

  if not include_sleep:
    time.sleep(1)
    qsize = len(es.timescale.point_queue)
    while qsize > 0:
      qsize = len(es.timescale.point_queue)
      logging.log_every_n_seconds(logging.INFO, 'Queue size %s', 2, qsize)
    es.timescale.stop_process_signal.value = True
    print(time.time())
    es.timescale.process.join(10)
    print(time.time())
  return es


def main(unused_argv):
  flags.mark_flag_as_required('filepath')
  ReplayLog(FLAGS.filepath, include_sleep=FLAGS.include_sleep)


if __name__ == '__main__':
  app.run(main)



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# pylint: disable=anomalous-backslash-in-string
"""Functions related to lap time calculations.

point_c |\
        | \
        |  \
        |   \
        |____\______ start/finish
        |    /
        |   /
        |  /
        |B/
point_b |/
        |
        |
        |
        |
        |
        |
        |
        |
point_a |
"""
# pylint: enable=anomalous-backslash-in-string
import math

from exit_speed import common_lib
from exit_speed import gps_pb2


def GetPriorUniquePoint(lap: gps_pb2.Lap,
                        point_c: gps_pb2.Point) -> gps_pb2.Point:
  """Avoids a division by zero if the two points have the same time.

  Older logged data had multiple points at the same time.
  """
  index = -1
  point = lap.points[-1]
  while point.time.ToNanoseconds() == point_c.time.ToNanoseconds():
    index -= 1
    point = lap.points[index]
  return point


def SolvePointBAngle(point_b, point_c) -> float:
  """Returns the angle of B."""
  dist_b_c = common_lib.PointDelta(point_b, point_c)
  # cos(B) = (c² + a² - b²)/2ca  https://rb.gy/pgi7zm
  a = point_b.start_finish_distance
  b = point_c.start_finish_distance
  c = dist_b_c
  return math.degrees(math.acos((c**2 + a**2 - b**2)/(2*c*a)))


def CalcAcceleration(point_b: gps_pb2.Point, point_c: gps_pb2.Point) -> float:
  """a = Δv/Δt"""
  return (((point_b.speed - point_c.speed) /
           (point_b.time.ToNanoseconds() - point_c.time.ToNanoseconds())) /
           1e-09)  # Nanoseconds > Seconds.


def PerpendicularDistanceToFinish(point_b_angle: float,
                                  point_b: gps_pb2.Point) -> float:
  """

  cos(B) = Adjacent / Hypotenuse
  https://www.mathsisfun.com/algebra/trig-finding-side-right-triangle.html
  """
  return math.cos(math.radians(point_b_angle)) * point_b.start_finish_distance


def SolveTimeToCrossFinish(point_b: gps_pb2.Point,
                           perp_dist_b: float,
                           accelration: float):
  """
  https://physics.stackexchange.com/questions/134771/deriving-time-from-acceleration-displacement-and-initial-velocity
  """
  sqrt = math.sqrt(point_b.speed ** 2 + 2 * accelration * perp_dist_b)
  return (point_b.speed * -1 + sqrt) / accelration


def GetTimeDelta(first_point, last_point) -> float:
  return last_point.time.ToNanoseconds() - first_point.time.ToNanoseconds()


def CalcTimeAfterFinish(lap: gps_pb2.Point) -> float:
  """Returns how many seconds between crossing start/finish and the last point.

  This assumes the first/last points of a lap are just past start/finish.
  """
  point_c = lap.points[-1]
  point_b = GetPriorUniquePoint(lap, point_c)
  point_b_angle = SolvePointBAngle(point_b, point_c)
  accelration = CalcAcceleration(point_b, point_c)
  perp_dist_b = PerpendicularDistanceToFinish(point_b_angle, point_b)
  time_to_fin = SolveTimeToCrossFinish(point_b, perp_dist_b, accelration)
  delta = GetTimeDelta(point_b, point_c)
  return delta - time_to_fin


def CalcLastLapDuration(session: gps_pb2.Session) -> float:
  """Calculates the last lap duration for the given session."""
  if len(session.laps) == 1:
    first_point = session.laps[0].points[0]
    last_point = session.laps[0].points[-1]
    return GetTimeDelta(first_point, last_point)
  prior_lap = session.laps[-2]
  current_lap = session.laps[-1]
  first_point = current_lap.points[0]
  last_point = current_lap.points[-1]
  delta = GetTimeDelta(first_point, last_point)
  prior_after = CalcTimeAfterFinish(prior_lap)
  current_after = CalcTimeAfterFinish(current_lap)
  return int(delta - current_after * 1e9 + prior_after * 1e9)



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unitests for gps_sensor.py"""
import multiprocessing
import unittest

import gps
import mock
from absl.testing import absltest

from exit_speed import gps_pb2
from exit_speed import gps_sensor


TEST_REPORT_VALUES = {
    u'epx': 7.409,
    u'epy': 8.266,
    u'epv': 20.01,
    u'ept': 0.005,
    u'lon': -2.1,
    u'eps': 165.32,
    u'lat': 14.2,
    u'track': 0.0,
    u'mode': 3,
    u'time': u'2019-12-19T05:24:24.100Z',
    u'device': u'/dev/ttyACM0',
    u'alt': 6.9,
    u'speed': 0.088,
    u'class': u'TPV'}
TEST_REPORT = gps.client.dictwrapper(TEST_REPORT_VALUES)


class TestGPSSensor(unittest.TestCase):
  """GPS Sensor unittests."""

  def testCheckReportFields(self):
    sensor = gps_sensor.GPS()
    with self.subTest(name='Populated Report'):
      self.assertTrue(sensor.CheckReportFields(TEST_REPORT))
    with self.subTest(name='Empty Report'):
      report = gps.client.dictwrapper({})
      self.assertFalse(sensor.CheckReportFields(report))

  def testGetReport(self):
    mock_gpsd = mock.create_autospec(gps.gps)
    sensor = gps_sensor.GPS(mock_gpsd)
    with self.subTest(name='Populated Report'):
      mock_gpsd.next.return_value = TEST_REPORT
      self.assertEqual(TEST_REPORT, sensor.GetReport())
      with self.subTest(name='Duplicate GPS report'):
        self.assertFalse(sensor.GetReport())
    with self.subTest(name='Empty Report'):
      report = gps.client.dictwrapper({})
      mock_gpsd.next.return_value = report
      self.assertFalse(sensor.GetReport())
    with self.subTest(name='Missing Req Field'):
      values = TEST_REPORT_VALUES.copy()
      del values['lat']
      report = gps.client.dictwrapper(values)
      mock_gpsd.next.return_value = report
      self.assertFalse(sensor.GetReport())

  def testGPSProcessLoop(self):
    point_queue = multiprocessing.Queue()
    with mock.patch.object(gps.gps, 'next') as mock_get:
      mock_get.return_value = TEST_REPORT
      proc = gps_sensor.GPSProcess({}, point_queue)
      while point_queue.empty():
        pass
      proc.Join()
      self.assertEqual(point_queue.qsize(), 1)
      point = gps_pb2.Point().FromString(point_queue.get())
      self.assertEqual(point.lat, TEST_REPORT_VALUES['lat'])
      self.assertEqual(point.lon, TEST_REPORT_VALUES['lon'])
      self.assertEqual(point.alt, TEST_REPORT_VALUES['alt'])
      self.assertEqual(point.speed, TEST_REPORT_VALUES['speed'])

if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Timescale interface for exporting data."""
import multiprocessing
import sys
import textwrap
import traceback
from typing import Optional
from typing import Text
from typing import Tuple

import psycopg2
from absl import flags
from absl import logging

from exit_speed import gps_pb2

FLAGS = flags.FLAGS
flags.DEFINE_string('timescale_db_spec',
                    'postgres://exit_speed:faster@localhost:/exit_speed',
                    'Postgres URI connection string.')
flags.DEFINE_integer('commit_cycle', 3,
                     'Number of points to commit at a time.')


SESSION_INSERT = textwrap.dedent("""
INSERT INTO sessions (time, track, car, live_data)
VALUES (%s, %s, %s, %s)
RETURNING id
""")
LAP_INSERT = textwrap.dedent("""
INSERT INTO laps (session_id, number)
VALUES (%s, %s)
RETURNING id
""")
LAP_DURATION_UPDATE = textwrap.dedent("""
UPDATE laps
SET duration_ms = %s
WHERE id = %s
""")
POINT_PREPARE = textwrap.dedent("""
PREPARE point_insert AS
INSERT INTO points (
    time, session_id, lap_id, lat, lon,
    alt, speed, geohash, elapsed_duration_ms, elapsed_distance_m,
    tps_voltage, water_temp_voltage, oil_pressure_voltage, rpm, afr,
    fuel_level_voltage, accelerometer_x, accelerometer_y, accelerometer_z,
    pitch, roll, gyro_x, gyro_y, gyro_z,
    front_brake_pressure_voltage, rear_brake_pressure_voltage,
    battery_voltage, oil_temp_voltage, labjack_temp_f,
    lf_tire_temp_inner, lf_tire_temp_middle, lf_tire_temp_outer,
    rf_tire_temp_inner, rf_tire_temp_middle, rf_tire_temp_outer,
    lr_tire_temp_inner, lr_tire_temp_middle, lr_tire_temp_outer,
    rr_tire_temp_inner, rr_tire_temp_middle, rr_tire_temp_outer
    )
VALUES ($1, $2, $3, $4, $5,
        $6, $7, $8, $9, $10,
        $11, $12, $13, $14, $15,
        $16, $17, $18, $19, $20,
        $21, $22, $23, $24, $25,
        $26, $27, $28, $29,
        $30, $31, $32,
        $33, $34, $35,
        $36, $37, $38,
        $39, $40, $41)
""")
POINT_INSERT = textwrap.dedent("""
EXECUTE point_insert (%s, %s, %s, %s, %s,
                      %s, %s, %s, %s, %s,
                      %s, %s, %s, %s, %s,
                      %s, %s, %s, %s, %s,
                      %s, %s, %s, %s, %s,
                      %s, %s, %s, %s,
                      %s, %s, %s,
                      %s, %s, %s,
                      %s, %s, %s,
                      %s, %s, %s)
""")


def ConnectToDB() -> psycopg2.extensions.connection:
  return psycopg2.connect(FLAGS.timescale_db_spec)


def GetConnWithPointPrepare(conn: psycopg2.extensions.connection =  None):
  conn = conn or ConnectToDB()
  with conn.cursor() as cursor:
    cursor.execute(POINT_PREPARE)
  return conn


class Timescale(object):
  """Interface for publishing data to timescale."""

  def __init__(self,
      car: Text,
      live_data: bool = True,
      start_process: bool = True):
    self.car = car
    self.live_data = live_data
    if start_process:
      self.process = multiprocessing.Process(target=self.Loop, daemon=True)
    self.stop_process_signal = multiprocessing.Value('b', False)
    self.manager = multiprocessing.Manager()
    self.timescale_conn = None
    self.session_time = None
    self.track = None
    self.session_id = None
    self.lap_number_ids = {}
    self.lap_queue = multiprocessing.Queue()
    self.lap_duration_queue = multiprocessing.Queue()
    self.point_queue = self.manager.list()  # Used as LifoQueue.
    self.retry_point_queue = []
    self.commit_cycle = 0

  def AddPointToQueue(self, point: gps_pb2.Point, lap_number: int):
    self.point_queue.append((point.SerializeToString(), lap_number))

  def ExportSession(self, cursor: psycopg2.extensions.cursor):
    if not self.session_id:
      args = (self.session_time.ToJsonString(), self.track, self.car,
              self.live_data)
      cursor.execute(SESSION_INSERT, args)
      self.session_id = cursor.fetchone()[0]
      self.timescale_conn.commit()

  def ExportLap(self, lap: gps_pb2.Lap, cursor: psycopg2.extensions.cursor):
    """Export the lap data to timescale."""
    args = (self.session_id, lap.number)
    cursor.execute(LAP_INSERT, args)
    self.lap_number_ids[lap.number] = cursor.fetchone()[0]
    self.timescale_conn.commit()

  def UpdateLapDuration(self,
                        lap_number: int,
                        duration: gps_pb2.Lap.duration,
                        cursor: psycopg2.extensions.cursor):
    """Exports lap duration to the Timescale backend."""
    args = (duration.ToMilliseconds(), self.lap_number_ids[lap_number])
    cursor.execute(LAP_DURATION_UPDATE, args)
    self.timescale_conn.commit()

  def ExportPoint(self,
                  point: gps_pb2.Point,
                  lap_number: int,
                  cursor: psycopg2.extensions.cursor):
    """Exports point data to timescale."""
    lap_id = self.lap_number_ids.get(lap_number)
    if lap_id:
      args = (point.time.ToJsonString(),
              self.session_id,
              lap_id,
              point.lat,
              point.lon,
              point.alt,
              point.speed * 2.23694,  # m/s to mph,
              point.geohash,
              point.elapsed_duration_ms,
              point.elapsed_distance_m,
              point.tps_voltage,
              point.water_temp_voltage,
              point.oil_pressure_voltage,
              point.rpm,
              point.afr,
              point.fuel_level_voltage,
              point.accelerometer_x,
              point.accelerometer_y,
              point.accelerometer_z,
              point.pitch,
              point.roll,
              point.gyro_x,
              point.gyro_y,
              point.gyro_z,
              point.front_brake_pressure_voltage,
              point.rear_brake_pressure_voltage,
              point.battery_voltage,
              point.oil_temp_voltage,
              point.labjack_temp_f,
              point.lf_tire_temp.inner,
              point.lf_tire_temp.middle,
              point.lf_tire_temp.outer,
              point.rf_tire_temp.inner,
              point.rf_tire_temp.middle,
              point.rf_tire_temp.outer,
              point.lr_tire_temp.inner,
              point.lr_tire_temp.middle,
              point.lr_tire_temp.outer,
              point.rr_tire_temp.inner,
              point.rr_tire_temp.middle,
              point.rr_tire_temp.outer,
              )
      cursor.execute(POINT_INSERT, args)
    else:
      self.retry_point_queue.append((point, lap_number))

  def GetLapFromQueue(self) -> Optional[gps_pb2.Lap]:
    if self.lap_queue.qsize() > 0:
      return gps_pb2.Lap().FromString(self.lap_queue.get())

  def GetLapDurationFromQueue(self) -> Optional[Tuple[int, int]]:
    if self.lap_duration_queue.qsize() > 0:
      return self.lap_duration_queue.get()

  def GetPointFromQueue(self) -> Optional[Tuple[gps_pb2.Point, int]]:
    """Blocks until a point is ready to export."""
    if self.retry_point_queue and not self.point_queue:
      return self.retry_point_queue.pop()
    # Queue is empty.
    while not self.point_queue:
      if self.stop_process_signal.value:
        return
    serialized_point, lap_number = self.point_queue.pop()
    return gps_pb2.Point().FromString(serialized_point), lap_number

  def _Commit(self):
    """Commits points to timescale based on FLAGS.commit_cycle."""
    if (self.commit_cycle >= FLAGS.commit_cycle or
        self.stop_process_signal.value):
      self.timescale_conn.commit()
      self.commit_cycle = 0
    else:
      self.commit_cycle += 1

  def Do(self):
    """One iteration of the infinite loop."""
    lap = None
    lap_number_und_duration = None
    point = None
    try:
      if not self.timescale_conn:
        self.timescale_conn = GetConnWithPointPrepare()
      lap = self.GetLapFromQueue()
      lap_number_und_duration = self.GetLapDurationFromQueue()
      point_und_lap_number = self.GetPointFromQueue()
      if point_und_lap_number:
        with self.timescale_conn.cursor() as cursor:
          self.ExportSession(cursor)
          if lap:
            self.ExportLap(lap, cursor)
          if lap_number_und_duration:
            self.UpdateLapDuration(lap_number_und_duration[0],
                                   lap_number_und_duration[1],
                                   cursor)
          self.ExportPoint(point_und_lap_number[0],
                           point_und_lap_number[1],
                           cursor)
          self._Commit()
    except psycopg2.Error:
      stack_trace = ''.join(traceback.format_exception(*sys.exc_info()))
      logging.log_every_n_seconds(logging.ERROR,
                                  'Error writing to timescale database\n%s',
                                  10,
                                  stack_trace)
      # Repopulate queues on errors.
      if lap:
        self.lap_queue.put(lap)
      if lap_number_und_duration:
        self.lap_duration_queue.put(lap_number_und_duration)
      if point:
        self.retry_point_queue.append(point_und_lap_number)
      self.timescale_conn = None  # Reset connection

  def Loop(self):
    """Tries to export point data to the timescale backend."""
    while not self.stop_process_signal.value:
      self.Do()
    if self.timescale_conn:
      self.timescale_conn.commit()

  def Start(self, session_time, track):
    self.session_time = session_time
    self.track = track
    self.process.start()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Database connection library."""
import os

import sqlalchemy

from exit_speed.dashboard import instance_manager
from exit_speed.dashboard import secret_manager

# Local args used when running a development instance.
# {'password': '',
#  'username': '',
#  'database': 'exit-speed',
#  'drivername': 'postgresql+psycopg2',
#  'host': 'ip-address'}
SECRET_LOCAL_ID = (
    'projects/794720490237/secrets/db-conn-local-args/versions/latest')


def InitPool() -> sqlalchemy.engine.base.Engine:
  if os.getenv('GOOGLE_CLOUD_PROJECT'):
    instance_manager.StartInstance()
    local_args = secret_manager.GetSecret(SECRET_LOCAL_ID)
    return sqlalchemy.create_engine(sqlalchemy.engine.url.URL(**local_args))
  return sqlalchemy.create_engine(sqlalchemy.engine.url.URL(
      **{'password': 'faster',
         'username': 'exit_speed',
         'database': 'exit_speed',
         'drivername': 'postgresql+psycopg2',
         'host': 'localhost'}))


POOL = InitPool()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Libaries for parsing yaml configs for Exit Speed."""
import yaml
from absl import flags

FLAGS = flags.FLAGS
flags.DEFINE_string('config_path', '/home/pi/git/exit_speed/etc/corrado.yaml',
                    'The location of the Exit Speed config.')


def LoadConfig(config_path=None):
  with open(config_path or FLAGS.config_path) as config_file:
    return yaml.load(config_file, Loader=yaml.BaseLoader)



#!/usr/bin/python3
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""App Engine dashboard using Dash, Plotly and Pandas."""
import urllib
from typing import Dict
from typing import List
from typing import Text
from typing import Tuple

import dash
import pandas as pd
import plotly.express as px
from absl import app as absl_app
from absl import flags
from dash import dash_table
from dash import dcc
from dash import html
from dash.dependencies import ALL
from dash.dependencies import Input
from dash.dependencies import Output
from dash.dependencies import State

from exit_speed.dashboard import queries

FLAGS = flags.FLAGS
flags.DEFINE_bool(
    'debug', True,
    'Set to true to enable auto reloading of Python code at the expense of '
    'high CPU load.')

app = dash.Dash(__name__)
server = app.server
SESSIONS = queries.GetSessions()
POINTS_COLUMNS = queries.GetPointsColumns() + [
    'front_brake_pressure_percentage',
    'rear_brake_pressure_percentage',
    'racing_line',
    'gsum',
    'time_delta'
    ]
TRACKS = queries.GetTracks()


app.layout = html.Div(
  style={'display': 'grid'},
  children=[
    dcc.Location(id='url', refresh=False),
    dcc.Link('Home', href='/'),
    dcc.Link('Clear', href='/track=None&points=None'),
    dcc.Dropdown(
      id='track-dropdown',
      options=[{'label': i, 'value': i} for i in TRACKS],
      searchable=False,
      clearable=False,
      style={'width': '50%'},
    ),
    dcc.Dropdown(
      id='points-dropdown',
      options=[{'label': i, 'value': i} for i in POINTS_COLUMNS],
      clearable=False,
      multi=True,
    ),
    dash_table.DataTable(
        id='sessions-table',
        columns=[
            {'name': i, 'id': i} for i in SESSIONS.columns
        ],
        filter_action='native',
        sort_action='native',
        sort_mode='single',
        sort_by=[{'column_id': 'lap_time',
                  'direction': 'asc'},
                 {'column_id': 'session_time',
                  'direction': 'desc'}],
        row_selectable='multi',
        page_action='native',
        page_current= 0,
        page_size= 10,
      ),
    html.Div(id='graphs'),
  ],
)

@app.callback(
  Output('url', 'href'),
  Input('url', 'href'),
  Input('track-dropdown', 'value'),
  Input('sessions-table', 'selected_row_ids'),  # lap_ids
  Input('points-dropdown', 'value'),
  prevent_initial_call=True,
)
def UpdateURL(
    href: Text,
    track: Text,
    lap_ids: List[int],
    points: List[Text],
  ) -> Text:
  args = {'track': track,
          'points': points}
  if lap_ids:
    args['lap_ids'] = lap_ids
  return urllib.parse.urljoin(href, urllib.parse.urlencode(args, doseq=True))


@app.callback(
  Output('track-dropdown', 'value'),
  Output('points-dropdown', 'value'),
  Output('sessions-table', 'selected_row_ids'),
  Input('url', 'pathname'),
)
def ParseURL(pathname: Text) -> Tuple[Text, List[Text], List[int]]:
  # Strip a leading "/" with [1:]
  params = urllib.parse.parse_qs(pathname[1:])
  url_track = params.get('track')
  if url_track:
    track = url_track[0]
  else:
    track = TRACKS[0]
  points = params.get(
              'points',
              ['racing_line',
               'time_delta',
               'speed',
               'tps_voltage',
               'front_brake_pressure_percentage',
               'gsum'])
  lap_ids = [int(lap_id) for lap_id in params.get('lap_ids', [])]
  return track, points, lap_ids


@app.callback(
  Output('sessions-table', 'data'),
  Input('track-dropdown', 'value'),
)
def UpdateSessions(track: pd.DataFrame) -> pd.DataFrame:
  filtered_df = SESSIONS[SESSIONS.track == track]
  return filtered_df.to_dict('records')


@app.callback(
  Output('graphs', 'children'),
  Input('sessions-table', 'selected_row_ids'),  # lap_ids
  Input('points-dropdown', 'value'),
)
def UpdateGraph(
    lap_ids: List[int], point_values: List[Text]) -> List[dcc.Graph]:
  if not isinstance(point_values, list):
    point_values = [point_values]
  if lap_ids:
    graphs = []
    laps_data = queries.GetLapsData(lap_ids, point_values)
    for point_value in point_values:
      figure_data = laps_data
      title = point_value
      # Copied so that time_delta can redefine without breaking other graphs.
      if point_value == 'time_delta':
        figure_data = laps_data.copy()
      if point_value == 'racing_line':
        graph_type = 'map'
        fig = px.line_geo(
            laps_data,
            title=title,
            lat='lat',
            lon='lon',
            color='lap_id',
            fitbounds='locations',
            )
      else:
        if point_value == 'time_delta':
          if len(lap_ids) < 2:
            continue  # Need at least two laps to make a comparison.
          figure_data = queries.GetTimeDelta(lap_ids)
          title = 'time_delta vs first selected lap (lap_id: %s)' % lap_ids[0]
        graph_type = 'graph'
        fig = px.line(
          figure_data,
          title=title,
          x='elapsed_distance_m',
          y=point_value,
          color='lap_id',
          hover_data=['lap_id', 'lap_number', point_value])
        fig.update_xaxes(showspikes=True)
        fig.update_yaxes(fixedrange=True)
        fig.update_layout(hovermode='x unified')
      graph = dcc.Graph({'type': graph_type,
                         'index': point_values.index(point_value)},
                        figure=fig,
                        style={'display': 'inline-grid', 'width': '50%'})
      graphs.append(graph)
    return graphs
  # Empty line when no rows have been selected.
  return [dcc.Graph(figure=px.line())]


@app.callback(
  Output({'type': 'graph', 'index': ALL}, 'relayoutData'),
  Output({'type': 'graph', 'index': ALL}, 'figure'),
  Input({'type': 'graph', 'index': ALL}, 'relayoutData'),
  State({'type': 'graph', 'index': ALL}, 'figure'))
def LinkedZoom(
    relayout_data: List[Dict],
    figure_states: List[Dict]) -> Tuple[List[Dict], List[Dict]]:
  unique_data = None
  for data in relayout_data:
    if relayout_data.count(data) == 1:
      unique_data = data
  if unique_data:
    for figure_state in figure_states:
      if unique_data.get('xaxis.autorange'):
        figure_state['layout']['xaxis']['autorange'] = True
      if unique_data.get('xaxis.range[0]'):
        figure_state['layout']['xaxis']['range'] = [
            unique_data['xaxis.range[0]'], unique_data['xaxis.range[1]']]
        figure_state['layout']['xaxis']['autorange'] = False
    return [unique_data] * len(relayout_data), figure_states
  return relayout_data, figure_states


def main(unused_argv):
  app.run_server(debug=FLAGS.debug)


if __name__ == '__main__':
  absl_app.run(main)



#!/usr/bin/python3
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Portland International Raceway."""
from exit_speed.tracks import base

PortlandInternationalRaceway = base.Track(
    name='Portland International Raceway',
    start_finish=(45.595015, -122.694526),
    turns=(
        #base.Turn('1', 45.596143, -122.698153),
        #base.Turn('2', 45.596494, -122.698325),
        #base.Turn('3', 45.596531, -122.699786),
        base.Turn('4', 45.597368, -122.702836),
        base.Turn('4a', 45.598057, -122.703058),
        base.Turn('5', 45.598724, -122.701839),
        base.Turn('6', 45.597977, -122.700049),
        base.Turn('7', 45.599913, -122.699313),
        base.Turn('8', 45.599559, -122.697438),
        base.Turn('9', 45.599445, -122.696173),
        base.Turn('10', 45.595370, -122.689487),
        base.Turn('11', 45.595004, -122.688507),
        base.Turn('12', 45.593828, -122.687975),
        ))



#!/usr/bin/python3
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Syncs local data to timescale."""
import os
from typing import List
from typing import Text

import psycopg2
import tracks
from absl import app
from absl import flags
from absl import logging

from exit_speed import cleanup_timescale
from exit_speed import data_logger
from exit_speed import replay_data
from exit_speed import timescale

FLAGS = flags.FLAGS
FLAGS.set_default('data_log_path', '/tmp')  # Dont clobber on replay.
FLAGS.set_default('led_brightness', 0.05)
FLAGS.set_default('commit_cycle', 1000)

LAP_LOG_PATH = '/home/pi/lap_logs'
SELECT_SESSION = 'SELECT * FROM sessions WHERE time = %s'


def GetLocalFiles() -> List:
  data_files = []
  for carname in os.listdir(LAP_LOG_PATH):
    for filename in os.listdir(os.path.join(LAP_LOG_PATH, carname)):
      if filename.endswith('.data'):
        data_files.append(os.path.join(LAP_LOG_PATH, carname, filename))
  return data_files


def IsFileAlreadySynced(timescale_conn: psycopg2.extensions.connection,
                        filepath: Text) -> bool:
  logger = data_logger.Logger(filepath)
  first_point = None
  for point in logger.ReadProtos():
    first_point = point
    print(first_point.time.ToJsonString())
    break
  else:
    return True  # Data file has no points.
  session_time = first_point.time.ToJsonString()
  cursor = timescale_conn.cursor()
  if cursor.execute(SELECT_SESSION, (session_time,)):
    return True
  if first_point:
    _, track, _ = tracks.FindClosestTrack(first_point)
    if track.name == 'Test Parking Lot':
      return True  # Skip the test parking lot.  Mostly development files.
  return False


def SyncFile(filepath):
  replay_data.ReplayLog(filepath, include_sleep=False)


def SyncLocalData():
  timescale_conn = timescale.ConnectToDB()
  for filepath in GetLocalFiles():
    logging.info('Syncing %s', filepath)
    if not IsFileAlreadySynced(timescale_conn, filepath):
      SyncFile(os.path.join(LAP_LOG_PATH, filepath))


def main(unused_argv):
  SyncLocalData()
  cleanup_timescale.CleanupTimescale()


if __name__ == '__main__':
  app.run(main)



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unitests for lab_lib.py"""
import unittest

import mock
from absl.testing import absltest

from exit_speed import common_lib
from exit_speed import gps_pb2
from exit_speed import lap_lib



class TestLapLib(unittest.TestCase):
  """Lap time unittests."""

  def testGetPriorUniquePoint(self):
    point_c = gps_pb2.Point()
    point_c.time.FromMilliseconds(10)
    lap = gps_pb2.Lap()
    point = lap.points.add()
    point.time.FromMilliseconds(9)
    point = lap.points.add()
    point.time.FromMilliseconds(9)
    point = lap.points.add()
    point.time.FromMilliseconds(8)
    returned = lap_lib.GetPriorUniquePoint(lap, point_c)
    self.assertEqual(8, returned.time.ToMilliseconds())

  def testSolvePointBAngle(self):
    point_b = gps_pb2.Point()
    point_b.start_finish_distance = 8
    point_c = gps_pb2.Point()
    point_c.start_finish_distance = 6
    with mock.patch.object(common_lib, 'PointDelta') as mock_delta:
      mock_delta.return_value = 7
      self.assertEqual(46.56746344221023,
                       lap_lib.SolvePointBAngle(point_b, point_c))

  def testCalcAcceleration(self):
    point_b = gps_pb2.Point()
    point_b.speed = 70
    point_b.time.FromMilliseconds(1)
    point_c = gps_pb2.Point()
    point_c.speed = 72
    point_c.time.FromMilliseconds(2)
    self.assertEqual(1999.9999999999998,
                     lap_lib.CalcAcceleration(point_b, point_c))

  def testPerpendicularDistanceToFinish(self):
    point_b_angle = 60
    point_b = gps_pb2.Point()
    point_b.start_finish_distance = 1000
    self.assertEqual(500.0000000000001,
                     lap_lib.PerpendicularDistanceToFinish(point_b_angle,
                                                           point_b))

  def testSolveTimeToCrossFinish(self):
    point_b = gps_pb2.Point()
    point_b.speed = 70
    acceleration = 2
    perp_dist_b = 1
    self.assertEqual(0.014282800023195819,
                     lap_lib.SolveTimeToCrossFinish(point_b,
                                                    perp_dist_b,
                                                    acceleration))

  def testGetTimeDelta(self):
    point_b = gps_pb2.Point()
    point_b.time.FromMilliseconds(1)
    point_c = gps_pb2.Point()
    point_c.time.FromMilliseconds(2)
    self.assertEqual(1 * 1e6, lap_lib.GetTimeDelta(point_b, point_c))

  def testCalcTimeAfterFinish(self):
    start_finish = gps_pb2.Point()
    start_finish.lat = 45.595015
    start_finish.lon = -122.694526
    lap = gps_pb2.Lap()
    point_b = lap.points.add()
    point_c = lap.points.add()
    point_b.time.FromMilliseconds(1)
    point_c.time.FromMilliseconds(2)
    point_b.lat = 45.594988
    point_b.lon = -122.694587
    point_c.lat = 45.595000
    point_c.lon = -122.694638
    point_b.start_finish_distance = common_lib.PointDelta(start_finish, point_b)
    point_c.start_finish_distance = common_lib.PointDelta(start_finish, point_c)
    point_b.speed = 70
    point_c.speed = 70.2
    self.assertEqual(1000000.0548742702, lap_lib.CalcTimeAfterFinish(lap))

  def testCalcLastLapDuration(self):
    start_finish = gps_pb2.Point()
    start_finish.lat = 45.595015
    start_finish.lon = -122.694526
    session = gps_pb2.Session()
    lap = session.laps.add()
    point_y = lap.points.add()
    point_z = lap.points.add()
    point_y.time.FromMilliseconds(1)
    point_z.time.FromMilliseconds(2)
    point_y.lat = 45.594988
    point_y.lon = -122.694587
    point_z.lat = 45.595000
    point_z.lon = -122.694638
    point_y.start_finish_distance = common_lib.PointDelta(start_finish, point_y)
    point_z.start_finish_distance = common_lib.PointDelta(start_finish, point_z)
    point_y.speed = 70
    point_z.speed = 70.2
    self.assertEqual(1.0 * 1e6, lap_lib.CalcLastLapDuration(session))

    lap = session.laps.add()
    point_a = point_z
    lap.points.append(point_a)
    point_b = lap.points.add()
    point_c = lap.points.add()
    point_b.time.FromMilliseconds(3)
    point_c.time.FromMilliseconds(4)
    point_b.lat = 45.594988
    point_b.lon = -122.694587
    point_c.lat = 45.595000
    point_c.lon = -122.694638
    point_b.start_finish_distance = common_lib.PointDelta(start_finish, point_b)
    point_c.start_finish_distance = common_lib.PointDelta(start_finish, point_c)
    point_b.speed = 70
    point_c.speed = 70.2
    self.assertEqual(2000000, lap_lib.CalcLastLapDuration(session))

if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Backups ~/lap_logs to GCS without writing to SD.

GOOGLE_APPLICATION_CREDENTIALS=<service>.json ./gcs_archive.py
"""
import datetime
import io
import tarfile

from absl import app
from absl import flags
from google.cloud import storage

FLAGS = flags.FLAGS
flags.DEFINE_string('data_log_path', '/home/pi/lap_logs',
                    'The directory containing lap logs.')
flags.DEFINE_string('gcs_bucket_name', 'lap_logs',
                    'The directory containing lap logs.')


def main(unused_argv):
  mem_file = io.BytesIO()
  with tarfile.open(fileobj=mem_file, mode='w:bz2') as tar:
    tar.add(FLAGS.data_log_path)
  mem_file.seek(0)
  storage_client = storage.Client()
  bucket = storage_client.bucket(FLAGS.gcs_bucket_name)
  today = datetime.date.today()
  blob_name = 'lap_logs_%s.tar.bz2' % today.strftime('%Y%m%d')
  blob = bucket.blob(blob_name)
  blob.upload_from_file(mem_file)


if __name__ == '__main__':
  app.run(main)



# pytype: skip-file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gps.proto
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='gps.proto',
  package='exit_speed',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\tgps.proto\x12\nexit_speed\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xc3\x06\n\x05Point\x12\x0b\n\x03lat\x18\x01 \x01(\x01\x12\x0b\n\x03lon\x18\x02 \x01(\x01\x12\x0b\n\x03\x61lt\x18\x03 \x01(\x01\x12\r\n\x05speed\x18\x04 \x01(\x01\x12(\n\x04time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1d\n\x15start_finish_distance\x18\x06 \x01(\x01\x12\x13\n\x0btps_voltage\x18\x07 \x01(\x01\x12\x1a\n\x12water_temp_voltage\x18\x08 \x01(\x01\x12\x1c\n\x14oil_pressure_voltage\x18\t \x01(\x01\x12\x0b\n\x03rpm\x18\n \x01(\x01\x12\x0b\n\x03\x61\x66r\x18\x0b \x01(\x01\x12\x1a\n\x12\x66uel_level_voltage\x18\x0c \x01(\x01\x12\x17\n\x0f\x61\x63\x63\x65lerometer_x\x18\r \x01(\x01\x12\x17\n\x0f\x61\x63\x63\x65lerometer_y\x18\x0e \x01(\x01\x12\x17\n\x0f\x61\x63\x63\x65lerometer_z\x18\x0f \x01(\x01\x12\r\n\x05pitch\x18\x10 \x01(\x01\x12\x0c\n\x04roll\x18\x11 \x01(\x01\x12\x0e\n\x06gyro_x\x18\x12 \x01(\x01\x12\x0e\n\x06gyro_y\x18\x13 \x01(\x01\x12\x0e\n\x06gyro_z\x18\x14 \x01(\x01\x12\x0f\n\x07geohash\x18\x15 \x01(\t\x12$\n\x1c\x66ront_brake_pressure_voltage\x18\x16 \x01(\x01\x12#\n\x1brear_brake_pressure_voltage\x18\x17 \x01(\x01\x12\x17\n\x0f\x62\x61ttery_voltage\x18\x18 \x01(\x01\x12\x18\n\x10oil_temp_voltage\x18\x19 \x01(\x01\x12\x1b\n\x13\x65lapsed_duration_ms\x18\x1a \x01(\x01\x12\x1a\n\x12\x65lapsed_distance_m\x18\x1b \x01(\x01\x12\x16\n\x0elabjack_temp_f\x18\x1c \x01(\x01\x12.\n\x0clf_tire_temp\x18\x1d \x01(\x0b\x32\x18.exit_speed.TireIrSensor\x12.\n\x0crf_tire_temp\x18\x1e \x01(\x0b\x32\x18.exit_speed.TireIrSensor\x12.\n\x0clr_tire_temp\x18\x1f \x01(\x0b\x32\x18.exit_speed.TireIrSensor\x12.\n\x0crr_tire_temp\x18  \x01(\x0b\x32\x18.exit_speed.TireIrSensor\"e\n\x03Lap\x12!\n\x06points\x18\x01 \x03(\x0b\x32\x11.exit_speed.Point\x12+\n\x08\x64uration\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x0e\n\x06number\x18\x03 \x01(\x05\"`\n\x07Session\x12\x1d\n\x04laps\x18\x01 \x03(\x0b\x32\x0f.exit_speed.Lap\x12\r\n\x05track\x18\x02 \x01(\t\x12\'\n\x0cstart_finish\x18\x03 \x01(\x0b\x32\x11.exit_speed.Point\"<\n\x0cTireIrSensor\x12\r\n\x05inner\x18\x01 \x01(\x01\x12\x0e\n\x06middle\x18\x02 \x01(\x01\x12\r\n\x05outer\x18\x03 \x01(\x01\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_POINT = _descriptor.Descriptor(
  name='Point',
  full_name='exit_speed.Point',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lat', full_name='exit_speed.Point.lat', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lon', full_name='exit_speed.Point.lon', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alt', full_name='exit_speed.Point.alt', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='speed', full_name='exit_speed.Point.speed', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='exit_speed.Point.time', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start_finish_distance', full_name='exit_speed.Point.start_finish_distance', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tps_voltage', full_name='exit_speed.Point.tps_voltage', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='water_temp_voltage', full_name='exit_speed.Point.water_temp_voltage', index=7,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='oil_pressure_voltage', full_name='exit_speed.Point.oil_pressure_voltage', index=8,
      number=9, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rpm', full_name='exit_speed.Point.rpm', index=9,
      number=10, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='afr', full_name='exit_speed.Point.afr', index=10,
      number=11, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fuel_level_voltage', full_name='exit_speed.Point.fuel_level_voltage', index=11,
      number=12, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='accelerometer_x', full_name='exit_speed.Point.accelerometer_x', index=12,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='accelerometer_y', full_name='exit_speed.Point.accelerometer_y', index=13,
      number=14, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='accelerometer_z', full_name='exit_speed.Point.accelerometer_z', index=14,
      number=15, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pitch', full_name='exit_speed.Point.pitch', index=15,
      number=16, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='roll', full_name='exit_speed.Point.roll', index=16,
      number=17, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gyro_x', full_name='exit_speed.Point.gyro_x', index=17,
      number=18, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gyro_y', full_name='exit_speed.Point.gyro_y', index=18,
      number=19, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gyro_z', full_name='exit_speed.Point.gyro_z', index=19,
      number=20, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='geohash', full_name='exit_speed.Point.geohash', index=20,
      number=21, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b''.decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='front_brake_pressure_voltage', full_name='exit_speed.Point.front_brake_pressure_voltage', index=21,
      number=22, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rear_brake_pressure_voltage', full_name='exit_speed.Point.rear_brake_pressure_voltage', index=22,
      number=23, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='battery_voltage', full_name='exit_speed.Point.battery_voltage', index=23,
      number=24, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='oil_temp_voltage', full_name='exit_speed.Point.oil_temp_voltage', index=24,
      number=25, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='elapsed_duration_ms', full_name='exit_speed.Point.elapsed_duration_ms', index=25,
      number=26, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='elapsed_distance_m', full_name='exit_speed.Point.elapsed_distance_m', index=26,
      number=27, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labjack_temp_f', full_name='exit_speed.Point.labjack_temp_f', index=27,
      number=28, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lf_tire_temp', full_name='exit_speed.Point.lf_tire_temp', index=28,
      number=29, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rf_tire_temp', full_name='exit_speed.Point.rf_tire_temp', index=29,
      number=30, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lr_tire_temp', full_name='exit_speed.Point.lr_tire_temp', index=30,
      number=31, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rr_tire_temp', full_name='exit_speed.Point.rr_tire_temp', index=31,
      number=32, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=91,
  serialized_end=926,
)


_LAP = _descriptor.Descriptor(
  name='Lap',
  full_name='exit_speed.Lap',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='exit_speed.Lap.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='duration', full_name='exit_speed.Lap.duration', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='number', full_name='exit_speed.Lap.number', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=928,
  serialized_end=1029,
)


_SESSION = _descriptor.Descriptor(
  name='Session',
  full_name='exit_speed.Session',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='laps', full_name='exit_speed.Session.laps', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='track', full_name='exit_speed.Session.track', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b''.decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start_finish', full_name='exit_speed.Session.start_finish', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1031,
  serialized_end=1127,
)


_TIREIRSENSOR = _descriptor.Descriptor(
  name='TireIrSensor',
  full_name='exit_speed.TireIrSensor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='inner', full_name='exit_speed.TireIrSensor.inner', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='middle', full_name='exit_speed.TireIrSensor.middle', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='outer', full_name='exit_speed.TireIrSensor.outer', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1129,
  serialized_end=1189,
)

_POINT.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_POINT.fields_by_name['lf_tire_temp'].message_type = _TIREIRSENSOR
_POINT.fields_by_name['rf_tire_temp'].message_type = _TIREIRSENSOR
_POINT.fields_by_name['lr_tire_temp'].message_type = _TIREIRSENSOR
_POINT.fields_by_name['rr_tire_temp'].message_type = _TIREIRSENSOR
_LAP.fields_by_name['points'].message_type = _POINT
_LAP.fields_by_name['duration'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_SESSION.fields_by_name['laps'].message_type = _LAP
_SESSION.fields_by_name['start_finish'].message_type = _POINT
DESCRIPTOR.message_types_by_name['Point'] = _POINT
DESCRIPTOR.message_types_by_name['Lap'] = _LAP
DESCRIPTOR.message_types_by_name['Session'] = _SESSION
DESCRIPTOR.message_types_by_name['TireIrSensor'] = _TIREIRSENSOR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Point = _reflection.GeneratedProtocolMessageType('Point', (_message.Message,), {
  'DESCRIPTOR' : _POINT,
  '__module__' : 'gps_pb2'
  # @@protoc_insertion_point(class_scope:exit_speed.Point)
  })
_sym_db.RegisterMessage(Point)

Lap = _reflection.GeneratedProtocolMessageType('Lap', (_message.Message,), {
  'DESCRIPTOR' : _LAP,
  '__module__' : 'gps_pb2'
  # @@protoc_insertion_point(class_scope:exit_speed.Lap)
  })
_sym_db.RegisterMessage(Lap)

Session = _reflection.GeneratedProtocolMessageType('Session', (_message.Message,), {
  'DESCRIPTOR' : _SESSION,
  '__module__' : 'gps_pb2'
  # @@protoc_insertion_point(class_scope:exit_speed.Session)
  })
_sym_db.RegisterMessage(Session)

TireIrSensor = _reflection.GeneratedProtocolMessageType('TireIrSensor', (_message.Message,), {
  'DESCRIPTOR' : _TIREIRSENSOR,
  '__module__' : 'gps_pb2'
  # @@protoc_insertion_point(class_scope:exit_speed.TireIrSensor)
  })
_sym_db.RegisterMessage(TireIrSensor)


# @@protoc_insertion_point(module_scope)



#!/usr/bin/python3
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unitests for tracks."""
import unittest

from absl.testing import absltest

from exit_speed import gps_pb2
from exit_speed import tracks

class TestTracks(unittest.TestCase):
  """Gyroscope unittests."""

  def testFindClosestTrack(self):
    point = gps_pb2.Point()
    point.lat = 45.595412
    point.lon = -122.693901
    distance, track, _ = tracks.FindClosestTrack(point)
    self.assertEqual(65.64651548636733, distance)
    self.assertEqual(track.name, 'Portland International Raceway')
    self.assertEqual(point.lat, 45.595412)
    self.assertEqual(point.lon, -122.693901)


if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common library unittest."""
import unittest

from absl.testing import absltest

from exit_speed import common_lib
from exit_speed import gps_pb2


class TestExitSpeed(unittest.TestCase):
  """Common library unittest."""

  def testPointDelta(self):
    point_a = gps_pb2.Point()
    point_b = gps_pb2.Point()
    point_a.lat = 1.1
    point_b.lat = 2.2
    point_a.lon = -1.1
    point_b.lon = -2.2
    self.assertEqual(171979.02735070087,
                     common_lib.PointDelta(point_a, point_b))


if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unitests for replay_data.py"""
import os
import sys
import unittest

import fake_rpi
import gps
import mock
import psycopg2
import testing.postgresql
from absl import flags
from absl.testing import absltest

from exit_speed import timescale
# pylint: disable=wrong-import-position
sys.modules['RPi'] = fake_rpi.RPi     # Fake RPi
sys.modules['RPi.GPIO'] = fake_rpi.RPi.GPIO # Fake GPIO
sys.modules['smbus'] = fake_rpi.smbus # Fake smbus (I2C)
# Fixes dotstar import on Travis. import fake_rpi
import adafruit_platformdetect
with mock.patch.object(adafruit_platformdetect, 'Detector') as mock_detector:
  mock_detector.chip.id.return_value = 'BCM2XXX'
  import adafruit_dotstar
  from exit_speed import replay_data
# pylint: enable=wrong-import-position

FLAGS = flags.FLAGS
FLAGS.set_default('config_path',
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
    'testdata/test_config.yaml'))
FLAGS.set_default('data_log_path', '/tmp')
FLAGS.set_default('filepath', '/dev/null')

Postgresql = testing.postgresql.PostgresqlFactory(cache_initialized_db=True)


def tearDownModule():
  Postgresql.clear_cache()


class TestReplayData(unittest.TestCase):
  """More of a end-to-end test for data in testdata/."""

  def setUp(self):
    super().setUp()
    self._AddMock(adafruit_dotstar, 'DotStar')
    self._AddMock(gps, 'gps')
    self.postgresql = Postgresql()
    with open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)),
        'testdata/timescale_schema')) as schema_file:
      statements = ''.join(schema_file.readlines())
    self.conn = psycopg2.connect(**self.postgresql.dsn())
    self.cursor = self.conn.cursor()
    self.cursor.execute(statements)
    self.conn.commit()
    mock_connect = self._AddMock(timescale, 'ConnectToDB')
    def _Connect():
      return psycopg2.connect(**self.postgresql.dsn())
    mock_connect.side_effect = _Connect

  def _AddMock(self, module, name):
    patch = mock.patch.object(module, name)
    self.addCleanup(patch.stop)
    return patch.start()

  def testEndToEnd(self):
    test_data_file = os.path.join(
            os.path.join(os.path.dirname(os.path.abspath(__file__)),
            'testdata/test_parking_lot_20hz_2020-06-11T22_1.data'))
    if test_data_file.endswith('.data'):
      with self.subTest(name=test_data_file):
        es = replay_data.ReplayLog(test_data_file)
        point_count = 0
        for lap in es.session.laps:
          point_count += len(lap.points)
        self.cursor.execute('SELECT count(*) FROM points')
        self.assertEqual(point_count, self.cursor.fetchone()[0])


if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""FXAS21002C gyroscope."""
import math
import time
from typing import Tuple

import adafruit_fxas21002c
import board
import busio
from absl import app

from exit_speed import gps_pb2
from exit_speed import sensor


class Gyroscope(object):
  """Measures rotational rate."""

  def __init__(self):
    self.i2c = busio.I2C(board.SCL, board.SDA)
    self.sensor = adafruit_fxas21002c.FXAS21002C(self.i2c)

  def GetRotationalValues(self) -> Tuple[float, float, float]:
    """Returns Gyroscope values in degrees."""
    gyro_x, gyro_y, gyro_z = self.sensor.gyroscope
    return (math.degrees(gyro_x),
            math.degrees(gyro_y),
            math.degrees(gyro_z))


class GyroscopeProcess(sensor.SensorBase):
  """Populates the SensorBase.point_queue with gyroscope values per loop."""

  def Loop(self):
    """Adds point data with gryoscope values to point queue."""
    gyro = Gyroscope()
    frequency_hz = int(
        self.config.get('gyroscope', {}).get('frequency_hz')) or 10
    while not self.stop_process_signal.value:
      cycle_time = time.time()
      x, y, z = gyro.GetRotationalValues()
      point = gps_pb2.Point()
      point.gyro_x = x
      point.gyro_y = y
      point.gyro_z = z
      self.AddPointToQueue(point)
      time.sleep(sensor.SleepBasedOnHertz(cycle_time, frequency_hz))



def main(unused_argv):
  gyro = Gyroscope()
  while True:
    x, y, z = gyro.GetRotationalValues()
    print('%.2f %.2f %.2f' % (x, y, z))
    time.sleep(1)


if __name__ == '__main__':
  app.run(main)



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Library for reading values from labjack."""
import multiprocessing
import sys
import time
import traceback
from typing import Dict

import u3
from absl import logging

from exit_speed import gps_pb2
from exit_speed import sensor


class Labjack(sensor.SensorBase):
  """Interface for the labjack DAQ."""
  HIGH_VOLTAGE_CHANNELS = (0, 1, 2, 3)

  def __init__(
      self,
      config: Dict,
      point_queue: multiprocessing.Queue,
      start_process:bool=True):
    self.u3 = None
    super().__init__(config, point_queue, start_process=start_process)

  def Set5vOutput(self):
    """In our case sets DAC0 to output 5v.

    The brake pressure sensors need a 5v reference signal.
    DAC is more stable than VS.
    """
    if self.config.get('labjack'):
      if self.config['labjack'].get('dac0_5v_out'):
        dac0_register = 5000
        self.u3.writeRegister(dac0_register, 5.0)

  def ReadValues(self):
    """Reads the labjack voltages."""
    try:
      if self.config.get('labjack'):
        point = gps_pb2.Point()
        point.labjack_temp_f = (
            self.u3.getTemperature() * 9.0/5.0 - 459.67)
        for input_name, proto_field in self.config['labjack'].items():
          if input_name.startswith('ain') or input_name.startswith('fio'):
            channel = int(input_name[-1])
            # Note getFeedback(u3.AIN(4)) is reading voltage from FIO4.
            # Physically AIN4 and FIO4 are identical.  AIN is for analog input
            # and FIO is flexible input/output.
            feedback = self.u3.getFeedback(u3.AIN(channel))[0]
            is_low_voltage = channel not in self.HIGH_VOLTAGE_CHANNELS
            voltage = self.u3.binaryToCalibratedAnalogVoltage(
                feedback,
                isLowVoltage=is_low_voltage,
                channelNumber=channel)
            if input_name in self.config['labjack'].get('tick_divider_10'):
              voltage = voltage * 10
            setattr(point, proto_field, voltage)
        self.AddPointToQueue(point)
    except u3.LabJackException:
      stack_trace = ''.join(traceback.format_exception(*sys.exc_info()))
      logging.log_every_n_seconds(logging.ERROR,
                                  'Error reading labjack values\n%s',
                                  10,
                                  stack_trace)

  def Loop(self):
    self.u3 = u3.U3()
    # AIN0-3 are always analog but we need to set FIO4-7 to analog before we can
    # read voltages on these terminals.
    self.u3.configIO(FIOAnalog = 0b11111111)
    self.Set5vOutput()
    frequency_hz = int(
        self.config.get('labjack', {}).get('frequency_hz')) or 10
    while not self.stop_process_signal.value:
      cycle_time = time.time()
      self.ReadValues()
      time.sleep(sensor.SleepBasedOnHertz(cycle_time, frequency_hz))



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The main script for starting exit speed."""
import datetime
import multiprocessing
import os

import sdnotify
import u3
from absl import app
from absl import flags
from absl import logging

from exit_speed import accelerometer
from exit_speed import common_lib
from exit_speed import config_lib
from exit_speed import data_logger
from exit_speed import gps_pb2
from exit_speed import gps_sensor
from exit_speed import gyroscope
from exit_speed import labjack
from exit_speed import lap_lib
from exit_speed import leds
from exit_speed import timescale
from exit_speed import tire_temperature
from exit_speed import tracks
from exit_speed import wbo2

FLAGS = flags.FLAGS
flags.DEFINE_string('data_log_path', '/home/pi/lap_logs',
                    'The directory to save data and logs.')


class ExitSpeed(object):
  """Main object which loops and logs data."""
  LABJACK_TIMER_CMD = u3.Timer0(UpdateReset=True, Value=0, Mode=None)
  REPORT_REQ_FIELDS = ('lat', 'lon', 'time', 'speed')

  def __init__(
      self,
      start_finish_range=20,  # Meters, ~4x the width of straightaways.
      live_data=True,
      min_points_per_session=60 * 10):  # 1 min @ gps 10hz):
    """Initializer.

    Args:
      start_finish_range: Maximum distance a point can be considered when
                          determining if the car crosses the start/finish.
      live_data: A boolean, if True indicates that this session's data should be
                 tagged as live.
      min_points_per_session:  Used to prevent sessions from prematurely ending.
    """
    self.start_finish_range = start_finish_range
    self.live_data = live_data
    self.min_points_per_session = min_points_per_session
    self.point_queue = multiprocessing.Queue()

    self.InitializeSubProcesses()
    self.leds = leds.LEDs()
    self.data_logger = None
    self.session = gps_pb2.Session()
    self.AddNewLap()
    self.point = None
    self.sdnotify = sdnotify.SystemdNotifier()
    self.sdnotify.notify('READY=1')

  def InitializeSubProcesses(self):
    """Initialize subprocess modules based on config.yaml."""
    self.config = config_lib.LoadConfig()
    if self.config.get('gps'):
      self.gps = gps_sensor.GPSProcess(self.config, self.point_queue)
      while self.point_queue.empty():
        logging.log_every_n_seconds(
            logging.INFO,
            'Waiting for GPS fix to determine track before starting other '
            'sensor subprocesses',
            10)
    if self.config.get('accelerometer'):
      self.accel = accelerometer.AccelerometerProcess(
          self.config, self.point_queue)
    if self.config.get('gyroscope'):
      self.gyro = gyroscope.GyroscopeProcess(self.config, self.point_queue)
    if self.config.get('labjack'):
      self.labjack = labjack.Labjack(self.config, self.point_queue)
    if self.config.get('tire_temps'):
      self.tire_temps = tire_temperature.MultiTireInterface(
          self.config, self.point_queue)
    if self.config.get('wbo2'):
      self.wbo2 = wbo2.WBO2(self.config, self.point_queue)
    if self.config.get('timescale'):
      car = self.config['car']
      logging.info('Logging for car: %s', car)
      self.timescale = timescale.Timescale(car, live_data=self.live_data)

  def AddNewLap(self) -> None:
    """Adds a new lap to the current session."""
    session = self.session
    lap = session.laps.add()
    self.lap = lap
    self.lap.number = len(session.laps)
    self.timescale.lap_queue.put(lap.SerializeToString())

  def GetLogFilePrefix(self, point: gps_pb2.Point, tz=None):
    utc_dt = point.time.ToDatetime()
    current_dt = utc_dt.replace(
        tzinfo=datetime.timezone.utc).astimezone(tz=tz)
    current_seconds = current_dt.second + current_dt.microsecond / 1e6
    return os.path.join(
        FLAGS.data_log_path,
        '%s/' % self.config.get('car', 'unknown_car'),
        '%s:%03f' % (current_dt.strftime('%Y-%m-%dT%H:%M'), current_seconds))

  def _InitializeDataLogger(self, point: gps_pb2.Point):
    file_prefix = self.GetLogFilePrefix(point)
    logging.info('Logging data to %s', file_prefix)
    self.data_logger = data_logger.Logger(file_prefix)

  def LogPoint(self) -> None:
    """Writes the current point to the data log."""
    point = self.point
    if not self.data_logger:
      self._InitializeDataLogger(point)
    self.data_logger.WriteProto(point)

  def CalculateElapsedValues(self):
    """Populates the elapsed_duration_ms and elapsed_distance_m point values."""
    point = self.point
    if len(self.lap.points) > 1:
      prior_point = lap_lib.GetPriorUniquePoint(self.lap, self.point)
      point.elapsed_duration_ms = (
          point.time.ToMilliseconds() -
          prior_point.time.ToMilliseconds() +
          prior_point.elapsed_duration_ms)
      point.elapsed_distance_m = (
          common_lib.PointDelta(point, prior_point) +
          prior_point.elapsed_distance_m)
    else:
      point.elapsed_duration_ms = 0
      point.elapsed_distance_m = 0

  def ProcessPoint(self) -> None:
    """Updates LEDs, logs point and writes data to PostgresSQL."""
    point = self.point
    point.start_finish_distance = common_lib.PointDelta(
        point, self.session.start_finish)
    self.leds.UpdateLeds(point)
    self.CalculateElapsedValues()
    self.LogPoint()
    self.timescale.AddPointToQueue(point, self.lap.number)

  def SetLapTime(self) -> None:
    """Sets the lap duration based on the first and last point time delta."""
    delta = lap_lib.CalcLastLapDuration(self.session)
    self.lap.duration.FromNanoseconds(delta)
    self.leds.SetBestLap(self.lap)
    self.timescale.lap_duration_queue.put((self.lap.number, self.lap.duration))
    minutes = self.lap.duration.ToSeconds() // 60
    seconds = (self.lap.duration.ToMilliseconds() % 60000) / 1000.0
    logging.info('New Lap %d:%.03f', minutes, seconds)

  def CrossStartFinish(self) -> None:
    """Checks and handles when the car crosses the start/finish."""
    if len(self.lap.points) >= self.min_points_per_session:
      prior_point = lap_lib.GetPriorUniquePoint(self.lap, self.point)
      if (self.point.start_finish_distance < self.start_finish_range and
          # First point past start/finish has an obtuse angle.
          lap_lib.SolvePointBAngle(prior_point, self.point) > 90):
        logging.info('Start/Finish')
        self.leds.CrossStartFinish()
        self.SetLapTime()
        self.AddNewLap()
        # Start and end laps on the same point just past start/finish.
        self.lap.points.append(prior_point)
        self.timescale.AddPointToQueue(prior_point, self.lap.number)
        # Reset elapsed values for first point of the lap.
        self.point.elapsed_duration_ms = 0
        self.point.elapsed_distance_m = 0
    self.lap.points.append(self.point)

  def ProcessLap(self) -> None:
    """Adds the point to the lap and checks if we crossed start/finish."""
    self.ProcessPoint()
    self.CrossStartFinish()

  def ProcessSession(self) -> None:
    """Start/ends the logging of data to log files."""
    if not self.session.track:
      _, track, start_finish = tracks.FindClosestTrack(self.point)
      logging.info('Closest track: %s', track.name)
      self.session.track = track.name
      self.session.start_finish.lat = start_finish.lat
      self.session.start_finish.lon = start_finish.lon
      self.timescale.Start(self.point.time, track.name)
    self.ProcessLap()

  def Run(self) -> None:
    """Runs exit speed in a loop."""
    while True:
      self.point = gps_pb2.FromString(self.point_queue.get())
      self.ProcessSession()
      logging.log_every_n_seconds(
          logging.INFO,
          'Point queue size currently at %d.',
          10,
          self.point_queue.qsize())
      self.sdnotify.notify(
          'STATUS=Last report time:%s' % self.point.time.ToJsonString())
      self.sdnotify.notify('WATCHDOG=1')


def main(unused_argv) -> None:
  logging.get_absl_handler().use_absl_log_file()
  es = None
  try:
    while True:
      logging.info('Starting Run')
      es = ExitSpeed()
      es.Run()
  except KeyboardInterrupt:
    logging.info('Keyboard interrupt')
  finally:
    if hasattr(es, 'point'):
      logging.info('Logging last point\n %s', es.point)
    logging.info('Done.\nExiting.')
    logging.exception('Ensure we log any exceptions')


if __name__ == '__main__':
  app.run(main)



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""MLX90640 InfraRed sensor.

https://github.com/melexis-fir/mlx9064x-driver-py
"""
import multiprocessing
import socket
import statistics
import struct
from typing import Dict
from typing import List
from typing import Text
from typing import Tuple

import numpy
from absl import app
from absl import flags
from absl import logging
from mlx import mlx90640

from exit_speed import gps_pb2
from exit_speed import sensor

FLAGS = flags.FLAGS
flags.DEFINE_string('ip_addr', None,
                    'IP address of the main Exit Speed process.')
flags.DEFINE_integer('port', None,
                     'Port number that corresponds to this corner.')


class InfraRedSensor(object):
  """Measures a 32x24 grid of temperature."""

  def __init__(self, com_port='I2C-1', i2c_addr=0x33, frame_rate=1.0,
      emissivity=0.9):
    self.frame_rate = frame_rate
    self.mlx = mlx90640.Mlx9064x(
        com_port, i2c_addr=i2c_addr, frame_rate=frame_rate)
    self.mlx.init()
    self.mlx.emissivity = emissivity

  def ReadFrame(self) -> List[float]:
    """Reads a frame from the infrared sensor."""
    try:
      frame = self.mlx.read_frame()
    except mlx90640.I2CAcknowledgeError:
      self.mlx.clear_error(self.frame_rate)
    frame = self.mlx.do_compensation(frame)
    frame = self.mlx.do_handle_bad_pixels(frame)
    logging.log_every_n_seconds(
        logging.INFO, 'Raw frame: \n%s', 60 * 5, frame)
    return frame

  def FormatFrame(self, frame: List[float]) -> List[List[float]]:
    """Formats a frame by pixel positions of rows and columns."""
    formatted = []
    for row_number in range(24):
      formatted.append(frame[row_number:row_number + 32])
    logging.log_every_n_seconds(
        logging.INFO, 'Formatted frame: \n%s', 60 * 5, formatted)
    return formatted


def FindTireIndex(jumps) -> int:
  first_jump = False
  index = 0
  for jump in jumps:
    if jump:
      first_jump = True
    if not jump and first_jump:
      return index
    index += 1
  return 0


class TireSensor(InfraRedSensor):
  """Tire temperature sensor."""
  # Temperature delta from tire edge to air/suspension.
  TEMP_EDGE_SENSITIVITY = 40

  def GetTempsByColumn(self) -> Dict[int, List[float]]:
    """Returns temperature in Celsius per column where key is column index."""
    formated_frame = self.FormatFrame(self.ReadFrame())
    column_temps = {}
    col_index = 0
    for row in formated_frame:
      for temp in row:
        column_temps.setdefault(col_index, []).append(temp)
      col_index += 1
    logging.log_every_n_seconds(
        logging.INFO, 'Column temps: \n%s', 60 * 5, column_temps)
    return column_temps

  def GetMedianColumnTemps(self) -> Dict[int, float]:
    """Returns the median temperature in Celsius per column."""
    column_temps = self.GetTempsByColumn()
    median_temps = {}
    for column, temps in column_temps.items():
      median_temps[column] = statistics.median(temps)
    logging.log_every_n_seconds(
        logging.INFO, 'Median temps: \n%s', 60 * 5, median_temps)
    return median_temps

  def GetTireTemps(self) -> Tuple[float, float, float]:
    """Attempts to find the edge of the tire based on temperature changes."""
    median_temps = self.GetMedianColumnTemps()
    diff = numpy.diff(list(median_temps.values()))
    jumps = abs(diff) > self.TEMP_EDGE_SENSITIVITY
    reverse_jumps = list(jumps)
    reverse_jumps.reverse()
    inner_tire_index = FindTireIndex(jumps)
    outer_tire_index = len(median_temps) - FindTireIndex(reverse_jumps) - 1
    middle_tire_index = round(statistics.mean(
        [inner_tire_index, outer_tire_index]))
    logging.log_every_n_seconds(
        logging.INFO, 'Inner index: %s, Middle index: %s, Outer index:%s',
        60 * 5, inner_tire_index, middle_tire_index, outer_tire_index,)
    return (
        median_temps[inner_tire_index],
        median_temps[middle_tire_index],
        median_temps[outer_tire_index])


class TireSensorClient(object):
  """Client for sending data from the Pi zeros."""

  def __init__(self, ip_addr: Text, port: int):
    self.ip_addr = ip_addr
    self.port = port
    self.sensor = TireSensor()
    self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

  def ReadAndSendData(self):
    temps = struct.pack('fff', *self.sensor.GetTireTemps())
    self.sock.sendto(temps, (self.ip_addr, self.port))

  def Loop(self):
    logging.info('Sending tire temperature data...')
    while True:
      self.ReadAndSendData()


class TireSensorServer(sensor.SensorBase):
  """Server for receiving data from the Pi zeros."""

  def __init__(
      self,
      corner: Text,
      ip_addr: Text,
      port: int,
      config: Dict,
      point_queue: multiprocessing.Queue,
      start_process: bool=True):
    self.corner = corner
    self.ip_addr = ip_addr
    self.port = port
    self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    super().__init__(config, point_queue, start_process=start_process)

  def Loop(self):
    self.sock.bind((self.ip_addr, self.port))
    while not self.stop_process_signal.value:
      data, _ = self.sock.recvfrom(1024)
      inside, middle, outside = struct.unpack('fff', data)
      point = gps_pb2.Point()
      corner = getattr(point, self.corner)
      setattr(corner, 'inner', (inside * 9/5) + 32)
      setattr(corner, 'middle', (middle * 9/5) + 32)
      setattr(corner, 'outer', (outside * 9/5) + 32)
      self.AddPointToQueue(point)


class MultiTireInterface(object):
  """Main class used by exit speed to hold each tire server."""

  def __init__(self, config: Dict, point_queue: multiprocessing.Queue):
    """Initializer.

    Args:
      config: A dict created from parsing the exit speed yaml config.
    """
    self.servers = {}
    for corner, ip_port in config['tire_temps'].items():
      self.servers[corner] = TireSensorServer(corner,
                                              ip_port['ip_addr'],
                                              int(ip_port['port']),
                                              config,
                                              point_queue)



def main(unused_argv):
  client = TireSensorClient(FLAGS.ip_addr, FLAGS.port)
  client.Loop()


if __name__ == '__main__':
  app.run(main)



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Converts a data log generated by exit speed to Traqmate's CSV format."""
import logging
import sys

from absl import app
from absl import flags

from exit_speed import csv_lib
from exit_speed import replay_data

FLAGS = flags.FLAGS


def main(unused_argv):
  logging.basicConfig(stream=sys.stdout, level=logging.INFO)
  es = replay_data.ReplayLog(FLAGS.filepath, include_sleep=False)
  new_path = '%s.csv' % FLAGS.filepath
  print('%s > %s' % (FLAGS.filepath, new_path))
  csv_lib.ConvertProtoToTraqmate(es.session, new_path)

if __name__ == '__main__':
  app.run(main)



#!/usr/bin/python3
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""GPS sensor."""
import geohash
import gps
from absl import app

from exit_speed import gps_pb2
from exit_speed import sensor

REPORT_REQ_FIELDS = ('lat', 'lon', 'time', 'speed')


class GPS(object):
  """Reads data from the GPS sensor."""

  def __init__(self, gpsd: gps.gps=None):
    self._last_gps_report_time = None  # Used to avoid duplicate GPS reports.
    self.gpsd = gpsd or gps.gps(mode=gps.WATCH_ENABLE|gps.WATCH_NEWSTYLE)

  def CheckReportFields(self, report: gps.client.dictwrapper) -> bool:
    """Verifies required report fields are present."""
    for field in REPORT_REQ_FIELDS:
      if not report.get(field):
        return False
    return True

  def GetReport(self) -> gps.client.dictwrapper:
    report = self.gpsd.next()
    if (report.get('class') == 'TPV' and self.CheckReportFields(report)):
      if (not self._last_gps_report_time or
          self._last_gps_report_time != report.time):
        self._last_gps_report_time = report.time
        return report


class GPSProcess(sensor.SensorBase):
  """Populates the SensorBase.point_queue with GPS values per loop."""

  def Loop(self):
    """Adds point data with GPS values to point queue."""
    gps_sensor = GPS()
    while not self.stop_process_signal.value:
      report = gps_sensor.GetReport()
      if report:
        point = gps_pb2.Point()
        point.lat = report.lat
        point.lon = report.lon
        if report.get('alt'):
          point.alt = report.alt
        point.speed = report.speed
        point.geohash = geohash.encode(point.lat, point.lon)
        self.AddPointToQueue(point)


def main(unused_argv):
  gps_sensor = GPS()
  while True:
    print(gps_sensor.GetReport())


if __name__ == '__main__':
  app.run(main)



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unitests for wbo2.py"""
import unittest

import mock
import serial
from absl.testing import absltest

from exit_speed import wbo2

TEST_FRAME = (b'Z\xa5\x08\x0c\xf8\x0f\xff \x00\x020\x01`\x03\xd0\x00\x15\x00'
              b'\x1a\x00 \x01\xa4\x00\x00\x03\x00i')


class TestWBO2(unittest.TestCase):
  """WBO2 unittests."""

  def setUp(self):
    super().setUp()
    self.start = 0

  def MockRead(self, size=1):
    data = TEST_FRAME[5:] + TEST_FRAME
    output = data[self.start:self.start + size]
    self.start += size
    return output

  def testFindFrameStart(self):
    mock_serial = mock.create_autospec(serial.Serial)
    mock_serial.read.side_effect = self.MockRead
    self.assertEqual(TEST_FRAME, wbo2.FindFrameStart(mock_serial))

  def testCheckFrame(self):
    self.assertTrue(wbo2.CheckFrame(TEST_FRAME))
    self.assertFalse(wbo2.CheckFrame(TEST_FRAME[:-1] + b'0x02'))

  def testReadSerial(self):
    mock_serial = mock.create_autospec(serial.Serial)
    mock_serial.read.side_effect = self.MockRead
    for frame in wbo2.ReadSerial(mock_serial):
      self.assertEqual(TEST_FRAME, frame)
      break

  def testGetBytes(self):
    self.assertEqual(0.5962854349951124, wbo2.GetBytes(TEST_FRAME, 'user_3'))
    self.assertEqual(0.0010162306553235967,
                     wbo2.GetBytes(TEST_FRAME, 'thermocouple_1'))
    self.assertEqual(3320, wbo2.GetBytes(TEST_FRAME, 'tick'))
    self.assertEqual(0, wbo2.GetBytes(TEST_FRAME, 'rpm_count'))
    self.assertEqual(14.69820556640625, wbo2.GetBytes(TEST_FRAME, 'lambda_16'))

  def testLambda16ToAFR(self):
    lambda_16 = int.from_bytes(b'\x0f\xff', 'big')
    self.assertEqual(14.69820556640625, wbo2.Lambda16ToAFR(lambda_16))

  def testRPMCountToRPM(self):
    self.assertEqual(6000, wbo2.RPMCountToRPM(1000, 4))
    self.assertEqual(4000, wbo2.RPMCountToRPM(1000, 6))

  def testGetUser3(self):
    self.assertEqual(0.5962854349951124,
            wbo2.GetBytes(TEST_FRAME, 'user_3'))


if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unittests for leds_test.py"""
import collections
import sys
import time
import unittest

import fake_rpi
import mock
from absl import flags
from absl.testing import absltest

from exit_speed import gps_pb2
sys.modules['RPi'] = fake_rpi.RPi     # Fake RPi
sys.modules['RPi.GPIO'] = fake_rpi.RPi.GPIO # Fake GPIO
sys.modules['smbus'] = fake_rpi.smbus # Fake smbus (I2C)
# pylint: disable=wrong-import-position
# Fixes dotstar import on Travis.
import adafruit_platformdetect
with mock.patch.object(adafruit_platformdetect, 'Detector') as mock_detector:
  mock_detector.chip.id.return_value = 'BCM2XXX'
  import adafruit_dotstar
  from exit_speed import leds
# pylint: enable=wrong-import-position

FLAGS = flags.FLAGS


class TestLEDs(unittest.TestCase):
  """LEDs unittest."""

  def setUp(self):
    super().setUp()
    mock_star = mock.create_autospec(adafruit_dotstar.DotStar)
    with mock.patch.object(adafruit_dotstar, 'DotStar') as mock_inst:
      mock_inst.return_value = mock_star
      self.leds = leds.LEDs()
    self.leds.last_led_update = time.time() - self.leds.led_update_interval
    self.mock_dots = mock.create_autospec(adafruit_dotstar.DotStar,
                                          spec_set=True)
    self.leds.dots = self.mock_dots

  def testLedInterval(self):
    self.assertTrue(self.leds.LedInterval())
    self.assertFalse(self.leds.LedInterval())
    time.sleep(self.leds.led_update_interval)
    self.leds.LedInterval(additional_delay=10)
    self.assertGreater(time.time() + 5, self.leds.led_update_interval)

  def testFill(self):
    color = (255, 255, 255)
    self.leds.Fill(color)
    self.mock_dots.fill.assert_called_once_with(color)
    self.mock_dots.fill.reset_mock()

    self.leds.Fill(color, ignore_update_interval=True)
    self.mock_dots.fill.assert_called_once_with(color)
    self.mock_dots.fill.reset_mock()

    self.leds.Fill(color, additional_delay=10, ignore_update_interval=True)
    self.assertGreater(time.time() + 5, self.leds.led_update_interval)
    self.mock_dots.fill.assert_called_once_with(color)

  def testFindNearestBestLapPoint(self):
    lap = gps_pb2.Lap()
    point = lap.points.add()
    point.lat = 1
    point.lon = 1
    point = lap.points.add()
    point.lat = 5
    point.lon = 5
    point = lap.points.add()
    point.lat = 20
    point.lon = 20

    self.leds.SetBestLap(lap)  # Build the tree.
    point = gps_pb2.Point()
    point.lat = 4
    point.lon = 4
    nearest = self.leds.FindNearestBestLapPoint(point)
    self.assertEqual(nearest.lat, 5)
    self.assertEqual(nearest.lon, 5)

  def testGetLedColor(self):
    self.leds.speed_deltas.extend([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
    self.assertTupleEqual(self.leds.GetLedColor(), (255, 0, 0))
    self.leds.speed_deltas.extend([0, -1, -2, -3, -4, -5, -6, -7, -8, -9])
    self.assertTupleEqual(self.leds.GetLedColor(), (0, 255, 0))
    self.leds.speed_deltas.extend([0, 1, 2, 3, 4, -5, -6, -7, -8, -9])
    self.assertTupleEqual(self.leds.GetLedColor(), (0, 255, 0))

  def testGetMovingSpeedDelta(self):
    self.leds.speed_deltas = [-100, 5, 100]
    self.assertEqual(5, self.leds.GetMovingSpeedDelta())

  def testUpdateSpeedDeltas(self):
    point = gps_pb2.Point()
    point.speed = 88  # mph
    best_point = gps_pb2.Point()
    best_point.speed = 87
    self.leds.UpdateSpeedDeltas(point, best_point)
    deltas = collections.deque(maxlen=FLAGS.speed_deltas)
    deltas.append(-1.0)
    self.assertSequenceEqual(deltas, self.leds.speed_deltas)

  @mock.patch.object(leds.LEDs, 'Fill')
  def testUpdateLeds(self, mock_fill):
    lap = gps_pb2.Lap()
    point = lap.points.add()
    point.speed = 88  # mph
    self.leds.UpdateLeds(point)
    self.assertFalse(mock_fill.mock_calls)  # No BallTree yet.

    self.leds.SetBestLap(lap)  # Used to build the BallTree.
    self.leds.UpdateLeds(point)
    color = (0, 255, 0)  # Green
    mock_fill.assert_called_once_with(color)
    deltas = collections.deque(maxlen=FLAGS.speed_deltas)
    deltas.append(0.0)
    self.assertSequenceEqual(deltas, self.leds.speed_deltas)

  def testSetBestLap(self):
    lap = gps_pb2.Lap()
    lap.duration.FromSeconds(100)
    point = lap.points.add()
    point.speed = 88  # mph

    self.leds.SetBestLap(lap)
    first_tree = self.leds.tree

    lap = gps_pb2.Lap()
    lap.duration.FromSeconds(99)
    point = lap.points.add()
    point.speed = 88  # mph
    self.leds.SetBestLap(lap)
    self.assertNotEqual(first_tree, self.leds.tree)

  @mock.patch.object(leds.LEDs, 'Fill')
  def testCrossStartFinish(self, mock_fill):
    self.leds.CrossStartFinish()
    blue = (0, 0, 255)
    mock_fill.assert_called_once_with(blue,
                                      additional_delay=1,
                                      ignore_update_interval=True)


if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unitests for timescale.py"""
import os
import unittest

import mock
import psycopg2
import testing.postgresql
from absl.testing import absltest

from exit_speed import gps_pb2
from exit_speed import timescale

Postgresql = testing.postgresql.PostgresqlFactory(cache_initialized_db=True)


def tearDownModule():
  Postgresql.clear_cache()


class TestTimescale(unittest.TestCase):
  """Timescale unittest."""

  def setUp(self):
    self.postgresql = Postgresql()
    schema_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        'testdata/timescale_schema')
    with open(schema_path) as schema_file:
      statements = ''.join(schema_file.readlines())
    conn = psycopg2.connect(**self.postgresql.dsn())
    cursor = conn.cursor()
    cursor.execute(statements)
    conn.commit()
    self.conn = timescale.GetConnWithPointPrepare(conn)
    self.cursor = self.conn.cursor()
    point = gps_pb2.Point()
    point.time.FromJsonString('2020-09-13T01:36:38.600Z')
    self.pusher = timescale.Timescale(
        'Corrado',
        start_process=False,
        )
    self.pusher.timescale_conn = self.conn
    self.pusher.track = 'Test Parking Lot'
    self.pusher.session_time = point.time

  def tearDown(self):
    self.cursor.close()
    self.conn.close()
    self.postgresql.stop()

  def testExportSession(self):
    self.pusher.ExportSession(self.cursor)
    self.assertEqual(1, self.pusher.session_id)

  def testExportLap(self):
    lap = gps_pb2.Lap()
    lap.number = 1
    self.pusher.ExportLap(lap, self.cursor)
    self.assertDictEqual({1: 1}, self.pusher.lap_number_ids)

  def testUpdateLapDuration(self):
    lap = gps_pb2.Lap()
    lap.duration.FromMilliseconds(90 * 1000)
    lap.number = 1
    self.pusher.ExportSession(self.cursor)
    self.pusher.ExportLap(lap, self.cursor)
    self.pusher.UpdateLapDuration(lap.number, lap.duration, self.cursor)
    self.cursor.execute('SELECT * FROM laps')
    self.conn.commit()
    lap_id, session_id, number, duration_ms = self.cursor.fetchone()
    self.assertEqual(1, lap_id)
    self.assertEqual(1, session_id)
    self.assertEqual(1, number)
    self.assertEqual(90000, duration_ms)

  def testExportPoint(self):
    lap = gps_pb2.Lap()
    lap.duration.FromMilliseconds(90 * 1000)
    lap.number = 1
    self.pusher.ExportSession(self.cursor)
    self.pusher.ExportLap(lap, self.cursor)
    self.pusher.UpdateLapDuration(lap.number, lap.duration, self.cursor)

    point = lap.points.add()
    point.alt = 1
    point.speed = 1
    point.lat = 45.69545832462609
    point.lon = -121.52551179751754
    point.tps_voltage = 2
    point.water_temp_voltage = 3
    point.oil_pressure_voltage = 4
    point.rpm = 1000
    point.afr = 14.7
    point.fuel_level_voltage = 5
    point.accelerometer_x = 0.0
    point.accelerometer_y = 1.7
    point.accelerometer_z = 1.2
    point.pitch = 0.2
    point.roll = 5.0
    point.gyro_x = 0.0
    point.gyro_y = 1.0
    point.gyro_z = 2.5
    point.geohash = 'c21efweg66fd'
    point.elapsed_duration_ms = 10
    point.elapsed_distance_m = 33
    point.front_brake_pressure_voltage = 3.5
    point.rear_brake_pressure_voltage = 2.5
    point.battery_voltage = 13.5
    point.oil_temp_voltage = 2.5
    point.labjack_temp_f = 77.9
    point.lf_tire_temp.inner = 170.0
    point.lf_tire_temp.middle = 180.0
    point.lf_tire_temp.outer = 200.0
    point.rf_tire_temp.inner = 190.0
    point.rf_tire_temp.middle = 190.0
    point.rf_tire_temp.outer = 190.0
    self.pusher.ExportPoint(point, 1, self.cursor)
    self.cursor.execute('SELECT * FROM points')
    (_, _, _, lat, lon, alt, speed, geohash, elapsed_duration_ms,
     elapsed_distance_m, tps_voltage, water_temp_voltage, oil_pressure_voltage,
     rpm, afr, fuel_level_voltage, accelerometer_x, accelerometer_y,
     accelerometer_z, pitch, roll, gyro_x, gyro_y, gyro_z,
     front_brake_pressure_voltage, rear_brake_pressure_voltage,
     battery_voltage, oil_temp_voltage, labjack_temp_f,
     lf_tire_temp_inner, lf_tire_temp_middle, lf_tire_temp_outer,
     rf_tire_temp_inner, rf_tire_temp_middle, rf_tire_temp_outer,
     lr_tire_temp_inner, lr_tire_temp_middle, lr_tire_temp_outer,
     rr_tire_temp_inner, rr_tire_temp_middle, rr_tire_temp_outer
     ) = self.cursor.fetchone()
    self.assertEqual(lat, 45.6954583246261)
    self.assertEqual(lon, -121.525511797518)
    self.assertEqual(alt, 1.0)
    self.assertEqual(speed, 2.23694)
    self.assertEqual(geohash, 'c21efweg66fd')
    self.assertEqual(elapsed_duration_ms, 10)
    self.assertEqual(elapsed_distance_m, 33)
    self.assertEqual(tps_voltage, 2.0)
    self.assertEqual(water_temp_voltage, 3.0)
    self.assertEqual(oil_pressure_voltage, 4.0)
    self.assertEqual(rpm, 1000)
    self.assertEqual(afr, 14.7)
    self.assertEqual(fuel_level_voltage, 5.0)
    self.assertEqual(accelerometer_x, 0.0)
    self.assertEqual(accelerometer_y, 1.7)
    self.assertEqual(accelerometer_z, 1.2)
    self.assertEqual(pitch, 0.2)
    self.assertEqual(roll, 5.0)
    self.assertEqual(gyro_x, 0.0)
    self.assertEqual(gyro_y, 1.0)
    self.assertEqual(gyro_z, 2.5)
    self.assertEqual(front_brake_pressure_voltage, 3.5)
    self.assertEqual(rear_brake_pressure_voltage, 2.5)
    self.assertEqual(battery_voltage, 13.5)
    self.assertEqual(oil_temp_voltage, 2.5)
    self.assertEqual(labjack_temp_f, 77.9)
    self.assertEqual(lf_tire_temp_inner, 170,0)
    self.assertEqual(lf_tire_temp_middle, 180.0)
    self.assertEqual(lf_tire_temp_outer, 200.0)
    self.assertEqual(rf_tire_temp_inner, 190.0)
    self.assertEqual(rf_tire_temp_middle, 190.0)
    self.assertEqual(rf_tire_temp_outer, 190.0)
    self.assertEqual(lr_tire_temp_inner, 0.0)
    self.assertEqual(lr_tire_temp_middle, 0.0)
    self.assertEqual(lr_tire_temp_outer, 0.0)
    self.assertEqual(rr_tire_temp_inner, 0.0)
    self.assertEqual(rr_tire_temp_middle, 0.0)
    self.assertEqual(rr_tire_temp_outer, 0.0)

  def testExportPointArrivesBeforeLap(self):
    point = gps_pb2.Point()
    self.pusher.ExportPoint(point, 99, self.cursor)
    self.assertEqual(1, len(self.pusher.retry_point_queue))

  def testGetPointFromQueue(self):
    with self.subTest(name='Success'):
      point = gps_pb2.Point()
      self.pusher.AddPointToQueue(point, 1)
      result = self.pusher.GetPointFromQueue()
      self.assertTrue(result)
      if result:
        returned_point, returned_lap_number = result
        self.assertEqual(point, returned_point)
        self.assertEqual(1, returned_lap_number)
    with self.subTest(name='Retry Queue'):
      point = gps_pb2.Point()
      self.pusher.retry_point_queue.append((point, 2))
      result = self.pusher.GetPointFromQueue()
      self.assertTrue(result)
      if result:
        returned_point, returned_lap_number = result
        self.assertEqual(point, returned_point)
        self.assertEqual(2, returned_lap_number)
    with self.subTest(name='Retry and Point Queue'):
      point = gps_pb2.Point()
      self.pusher.AddPointToQueue(point, 1)
      point = gps_pb2.Point()
      self.pusher.retry_point_queue.append((point, 2))
      result = self.pusher.GetPointFromQueue()
      self.assertTrue(result)
      if result:
        returned_point, returned_lap_number = result
        self.assertEqual(point, returned_point)
        self.assertEqual(1, returned_lap_number)

  @mock.patch.object(timescale, 'ConnectToDB')
  def testDo(self, mock_conn):
    mock_conn.return_value = self.conn
    lap = gps_pb2.Lap()
    lap.duration.FromMilliseconds(90 * 1000)
    lap.number = 1
    point = lap.points.add()
    point.alt = 1
    point.speed = 1
    point.lat = 45.69545832462609
    point.lon = -121.52551179751754
    point.tps_voltage = 2
    point.water_temp_voltage = 3
    point.oil_pressure_voltage = 4
    point.rpm = 1000
    point.afr = 14.7
    point.fuel_level_voltage = 5
    point.accelerometer_x = 0.0
    point.accelerometer_y = 1.7
    point.accelerometer_z = 1.2
    self.pusher.lap_queue.put(lap.SerializeToString())
    self.pusher.AddPointToQueue(point, 1)
    with self.subTest(name='Success'):
      self.pusher.Do()
      self.cursor.execute('SELECT count(*) FROM sessions')
      self.assertEqual(1, self.cursor.fetchone()[0])
      self.cursor.execute('SELECT count(*) FROM laps')
      self.assertEqual(1, self.cursor.fetchone()[0])
      self.cursor.execute('SELECT count(*) FROM points')
      self.assertEqual(1, self.cursor.fetchone()[0])
      self.assertEqual(0, self.pusher.lap_queue.qsize())
      self.assertEqual(0, self.pusher.lap_duration_queue.qsize())
      self.assertEqual(0, len(self.pusher.point_queue))
    with self.subTest(name='Second Point'):
      self.pusher.AddPointToQueue(point, 1)
      self.pusher.Do()
      self.cursor.execute('SELECT count(*) FROM sessions')
      self.assertEqual(1, self.cursor.fetchone()[0])
      self.cursor.execute('SELECT count(*) FROM laps')
      self.assertEqual(1, self.cursor.fetchone()[0])
      self.cursor.execute('SELECT count(*) FROM points')
      self.assertEqual(2, self.cursor.fetchone()[0])
      self.assertEqual(0, self.pusher.lap_queue.qsize())
      self.assertEqual(0, self.pusher.lap_duration_queue.qsize())
      self.assertEqual(0, len(self.pusher.point_queue))
    with self.subTest(name='Lap Duration'):
      self.pusher.lap_duration_queue.put((1, lap.duration))
      self.pusher.AddPointToQueue(point, 1)
      self.pusher.Do()
      self.assertEqual(0, self.pusher.lap_queue.qsize())
      self.assertEqual(0, self.pusher.lap_duration_queue.qsize())
      self.assertEqual(0, len(self.pusher.point_queue))
    with self.subTest(name='Point Too Early'):
      self.pusher.point_queue.append((point.SerializeToString(), 2))
      self.pusher.Do()
      self.cursor.execute('SELECT count(*) FROM sessions')
      self.assertEqual(1, self.cursor.fetchone()[0])
      self.cursor.execute('SELECT count(*) FROM laps')
      self.assertEqual(1, self.cursor.fetchone()[0])
      self.cursor.execute('SELECT count(*) FROM points')
      self.assertEqual(3, self.cursor.fetchone()[0])
      self.assertEqual(1, len(self.pusher.retry_point_queue))
      self.assertEqual(0, self.pusher.lap_queue.qsize())
      self.assertEqual(0, self.pusher.lap_duration_queue.qsize())
      self.assertEqual(1, len(self.pusher.retry_point_queue))
    with self.subTest(name='Exception'):
      lap.number = 2
      lap.duration.FromMilliseconds(90 * 1000)
      self.pusher.lap_queue.put(lap.SerializeToString())
      self.pusher.lap_duration_queue.put((1, lap.duration))
      self.pusher.AddPointToQueue(point, 1)
      with mock.patch.object(self.pusher, 'ExportPoint') as mock_export:
        mock_export.side_effect = psycopg2.Error
        self.pusher.Do()
        self.assertEqual(1, self.pusher.lap_queue.qsize())
        self.assertEqual(1, self.pusher.lap_duration_queue.qsize())
        self.assertEqual(1, len(self.pusher.retry_point_queue))

  def testDoCommitCycle(self):
    """Ensures points aren't dropped if errrors arrive between commits."""
    point = gps_pb2.Point()
    point.alt = 1
    point.speed = 1
    point.lat = 45.69545832462609
    point.lon = -121.52551179751754
    point.tps_voltage = 2
    point.water_temp_voltage = 3
    point.oil_pressure_voltage = 4
    point.rpm = 1000
    point.afr = 14.7
    point.fuel_level_voltage = 5
    point.accelerometer_x = 0.0
    point.accelerometer_y = 1.7
    point.accelerometer_z = 1.2
    self.assertEqual(0, len(self.pusher.point_queue))
    self.pusher.AddPointToQueue(point, 1)
    self.pusher.Do()
    self.pusher.AddPointToQueue(point, 1)
    self.pusher.Do()
    with mock.patch.object(self.pusher, '_Commit') as mock_commit:
      mock_commit.side_effect = psycopg2.Error
      self.pusher.AddPointToQueue(point, 1)
      self.pusher.Do()
    self.assertEqual(3, len(self.pusher.retry_point_queue))


if __name__ == '__main__':
  absltest.main()



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Controls the Adafruit LEDs."""
import collections
import statistics
import time
from typing import Tuple

import adafruit_dotstar
import board
import numpy as np
from absl import flags
from absl import logging
from gps import EarthDistanceSmall
from sklearn.neighbors import BallTree

from exit_speed import gps_pb2

FLAGS = flags.FLAGS
flags.DEFINE_float('led_brightness', 0.5,
                   'Percentage of how bright the LEDs are. IE 0.5 == 50%.')
flags.DEFINE_float('led_update_interval', 0.2,
                   'Limits how often the LEDs are able to change to prevent '
                   'excessive flickering.')
flags.DEFINE_integer('speed_deltas', 10,
                     'Used to smooth out GPS data.  This controls how many '
                     'recent speed deltas are stored.  50 at 10hz means a '
                     'median of the last 5 seconds is used.')


class LEDs(object):
  """Interface for controlling the changing of the LED colors."""

  def __init__(self):
    self.led_update_interval = FLAGS.led_update_interval
    self.last_led_update = time.time()
    self.dots = adafruit_dotstar.DotStar(board.SCK, board.MOSI, 10,
                                         brightness=FLAGS.led_brightness)
    self.Fill((0, 0, 255), ignore_update_interval=True)  # Blue
    self.tree = None
    self.speed_deltas = collections.deque(maxlen=FLAGS.speed_deltas)
    self.best_lap = None

  def LedInterval(self,
                  additional_delay: float = 0) -> bool:
    """Returns True if it is safe to update the LEDs based on interval."""
    now = time.time()
    if now - self.last_led_update > self.led_update_interval:
      self.last_led_update = now + additional_delay
      return True
    return False

  def Fill(self,
           color: Tuple[int, int, int],
           additional_delay: float = 0.0,
           ignore_update_interval: bool = False) -> None:
    """Sets all of the LEDs to the specified color.

    Args:
      color: A tuple of (R, G, B) values 0-255.
      additional_delay: Adds an additional delay to the update interval.
                        This is used when crossing start/finish to set the LEDs
                        to blue for a full second.
      ignore_update_interval: If True skips the update interval check.
    """
    update = self.LedInterval(additional_delay)
    if ignore_update_interval or update:
      self.dots.fill(color)

  def FindNearestBestLapPoint(self, point: gps_pb2.Point) -> gps_pb2.Point:
    """Returns the nearest point on the best lap to the given point."""
    neighbors = self.tree.query([[point.lat, point.lon]], k=1,
                                return_distance=False)
    for neighbor in neighbors[0]:
      x = self.tree.data[:, 0][neighbor]
      y = self.tree.data[:, 1][neighbor]
      for point_b in self.best_lap.points:
        if point_b.lat == x and point_b.lon == y:
          return point_b

  def GetLedColor(self) -> Tuple[int, int, int]:
    median_delta = self.GetMovingSpeedDelta()
    if median_delta > 0:
      return (255, 0, 0)  # Red
    return (0, 255, 0)  # Green

  def GetMovingSpeedDelta(self) -> float:
    """Returns the median speed delta over a time period based on the ring size.

    This helps smooth out the LEDs a bit so they're not so flickery by looking
    a moving median of the speed deltas.  Ring size controls how big the ring
    buffer can be, IE the number of deltas to hold on to.  At a GPS singal of
    10hz a ring size of 10 will cover a second worth of time.
    """
    return statistics.median(self.speed_deltas)

  def UpdateSpeedDeltas(self,
                        point: gps_pb2.Point,
                        best_point: gps_pb2.Point) -> float:
    speed_delta = best_point.speed - point.speed
    self.speed_deltas.append(speed_delta)
    return statistics.median(self.speed_deltas)

  def UpdateLeds(self, point: gps_pb2.Point) -> None:
    """Update LEDs based on speed difference to the best lap."""
    if self.tree:
      best_point = self.FindNearestBestLapPoint(point)
      self.UpdateSpeedDeltas(point, best_point)
      led_color = self.GetLedColor()
      self.Fill(led_color)

  def SetBestLap(self, lap: gps_pb2.Lap) -> None:
    """Sets best lap and builds a KDTree for finding closest points."""
    if (not self.best_lap or
        lap.duration.ToNanoseconds() < self.best_lap.duration.ToNanoseconds()):
      minutes = lap.duration.ToSeconds() // 60
      seconds = (lap.duration.ToMilliseconds() % 60000) / 1000.0
      logging.info('New Best Lap %d:%.03f', minutes, seconds)
      self.best_lap = lap
      x_y_points = []
      for point in lap.points:
        x_y_points.append([point.lat, point.lon])
      self.tree = BallTree(np.array(x_y_points), leaf_size=30,
                           metric='pyfunc', func=EarthDistanceSmall)

  def CrossStartFinish(self) -> None:
    self.Fill((0, 0, 255),  # Blue
              additional_delay=1,
              ignore_update_interval=True)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Library for starting/stopping SQL & VM instances."""
import logging

from googleapiclient import discovery

from exit_speed.dashboard import secret_manager

# Obscuring the project id.  This is probably overkill but this code is public.
SECRET_PROJECT_ID = (
    'projects/794720490237/secrets/project_id/versions/latest')
INSTANCE_NAME = 'exit-speed'


def IsSQLInstanceUp() -> bool:
  service = discovery.build('sqladmin', 'v1beta4')
  project = secret_manager.GetSecret(SECRET_PROJECT_ID)
  request = service.instances().get(project=project, instance=INSTANCE_NAME)
  response = request.execute()
  return response['settings']['activationPolicy'] == 'ALWAYS'


def StartInstance():
  service = discovery.build('sqladmin', 'v1beta4')
  project = secret_manager.GetSecret(SECRET_PROJECT_ID)
  request = service.instances().get(project=project, instance=INSTANCE_NAME)
  response = request.execute()
  if IsSQLInstanceUp():
    logging.info('SQL instance is already up')
  else:
    request = service.instances().patch(
        project=project, instance=INSTANCE_NAME, body={
            'settings': {'activationPolicy': 'ALWAYS'}})
    response = request.execute()
    logging.info('Start SQL instance response: %s', response)



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Library for appending protos to a file.

Based the recommendation here.
https://developers.google.com/protocol-buffers/docs/techniques
"""
import glob
import os
import re
from typing import Generator
from typing import Text

from absl import logging
from google.protobuf import message

from exit_speed import gps_pb2

BYTE_ORDER = 'big'

class Error(Exception):
  """Base module exception."""


class UnableToDetermineProtoLength(Error):
  """Raised when unable to determine proto length from filename."""


class Logger(object):
  """Interface for writing and reading protos to disk."""

  def __init__(self, file_prefix_or_name: Text):
    """Initializer.

    Args:
      file_prefix: File prefix with out the .data extension.
                   Ex: testdata/PIR_race_10hz_gps_only_2020-06-21
                   Or the a file's complete name including the _#.data suffix.
    """
    super().__init__()
    if file_prefix_or_name.endswith('.data'):
      self.file_prefix = file_prefix_or_name[:-7]  # Chop off the _1.data
      logging.info('file_prefix: %s', self.file_prefix)
    else:
      self.file_prefix = file_prefix_or_name
    self.file_path = None
    self.current_file = None
    self.current_proto_len = 1
    self._SetFilePath()

  def __del__(self):
    if self.current_file:
      self.current_file.close()

  def _SetFilePath(self):
    self.file_path = '%s_%s.data' % (self.file_prefix, self.current_proto_len)

  def _SetCurrentFile(self):
    if self.current_file:
      self.current_file.flush()
    parent_dir = os.path.dirname(self.file_path)
    if not os.path.exists(parent_dir):
      os.mkdir(parent_dir)
    self.current_file = open(self.file_path, 'wb')

  def GetFile(self, proto_len):
    if proto_len < int.from_bytes(b'\xff' * self.current_proto_len, 'big'):
      if not self.current_file:
        self._SetCurrentFile()
    else:
      self.current_proto_len += 1
      self._SetFilePath()
      self._SetCurrentFile()
    return self.current_file

  def WriteProto(self, proto: gps_pb2.Point):
    proto_bytes = proto.SerializePartialToString()
    proto_len = len(proto_bytes)
    data_file = self.GetFile(proto_len)
    data_file.write(proto_len.to_bytes(
      self.current_proto_len, BYTE_ORDER) + proto_bytes)

  def ReadProtos(self) -> Generator[gps_pb2.Point, None, None]:
    files_to_read = glob.glob(self.file_prefix + '*.data')
    logging.info('Data files to read: %s', ','.join(files_to_read))
    for file_path in files_to_read:
      self.file_path = file_path
      match = re.match(r'.*(\d)\.data', file_path)
      if not match:
        raise UnableToDetermineProtoLength(
            'Failed to parse proto length from filename.  Files should end in '
            'the format _#.data where # denotes the proto length. \n'
            'Filepath: %s' % file_path)
      self.current_proto_len = int(match.groups()[0])
      with open(self.file_path, 'rb') as data_file:
        while True:
          proto_len = int.from_bytes(
              data_file.read(self.current_proto_len), BYTE_ORDER)
          proto_bytes = data_file.read(proto_len)
          if proto_len and proto_bytes:
            try:
              point = gps_pb2.Point.FromString(proto_bytes)
              yield point
            except message.DecodeError:
              logging.info('Decode error.  Likely the file writes were '
                           'interrupted.  Treating as end of file.')
              break
          else:
            break



#!/usr/bin/python3
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Track list and helper functions."""
from typing import Tuple

from exit_speed import common_lib
from exit_speed import gps_pb2
from exit_speed.tracks import area27
from exit_speed.tracks import base
from exit_speed.tracks import oregon_raceway_park
from exit_speed.tracks import pacific_raceways
from exit_speed.tracks import portland_internal_raceways
from exit_speed.tracks import spokane_raceway
from exit_speed.tracks import test_track
from exit_speed.tracks import the_ridge

TRACK_LIST = (
    area27.Area27,
    oregon_raceway_park.OregonRacewayPark,
    pacific_raceways.PacificRaceways,
    portland_internal_raceways.PortlandInternationalRaceway,
    spokane_raceway.SpokaneRaceway,
    test_track.TestTrack,
    the_ridge.TheRidge,
    )


def FindClosestTrack(
    point: gps_pb2.Point) -> Tuple[float, base.Track, gps_pb2.Point]:
  """Returns the distance, track and start/finish of the closest track."""
  distance_track = []
  for track in TRACK_LIST:
    lat, lon = track.start_finish
    track_point = gps_pb2.Point()
    track_point.lat = lat
    track_point.lon = lon
    distance = common_lib.PointDelta(point, track_point)
    distance_track.append((distance, track, track_point))
  return sorted(distance_track)[0]



#!/usr/bin/python3
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""FXOS8700 accelerometer."""
import math
import time
from typing import Text
from typing import Tuple

import adafruit_fxos8700
import board
import busio
from absl import app

from exit_speed import gps_pb2
from exit_speed import sensor


class Accelerometer(object):
  """Measures the G forces the car experiences."""
  GRAVITY = 9.81
  X_LOW = -9.7459664498
  Y_LOW = -9.8632147572
  Z_LOW = -9.4660062056
  X_HIGH = 10.420742422999998
  Y_HIGH = 9.164510558
  Z_HIGH = 10.301101292999999

  def __init__(self):
    self.i2c = busio.I2C(board.SCL, board.SDA)
    self.accelerometer = adafruit_fxos8700.FXOS8700(
        self.i2c, accel_range=adafruit_fxos8700.ACCEL_RANGE_8G)

  def CorrectValue(self, axis: Text, value: float) -> float:
    """Returns the values after correcting for calibration.

    Args:
      axis: A string of the axis such as x,y,z.
      value: A value from self.accelerometer.accelerometer

    https://learn.sparkfun.com/tutorials/adxl345-hookup-guide#calibration
    https://www.nxp.com/docs/en/application-note/AN4399.pdf
    """
    low = getattr(self, '%s_LOW' % axis.upper())
    high = getattr(self, '%s_HIGH' % axis.upper())
    offset = 0.5 * (high + low)
    gain = 0.5 * ((high - low) / 9.81)
    return (value - offset) / gain

  def GetGForces(self) -> Tuple[float, float, float]:
    """Returns the number of G forces measured in each direction."""
    x, y, z = self.accelerometer.accelerometer
    x = self.CorrectValue('x', x)
    y = self.CorrectValue('y', y)
    z = self.CorrectValue('z', z)
    return x / self.GRAVITY, y / self.GRAVITY, z / self.GRAVITY

  def CalcPitchAndRoll(self, x_gs, y_gs, z_gs) -> Tuple[float, float]:
    """Calculates the pitch and roll based on the G readings."""
    pitch = ((math.atan2(x_gs, math.sqrt(y_gs * y_gs + z_gs * z_gs)) * 180) /
        math.pi)
    roll = (math.atan2(-y_gs, z_gs) * 180)  / math.pi
    return pitch, roll


class AccelerometerProcess(sensor.SensorBase):
  """Populates the SensorBase.point_queue with accelerometer values per loop."""

  def Loop(self):
    """Adds point data with accelerometer values to point queue."""
    accel = Accelerometer()
    frequency_hz = int(
        self.config.get('accelerometer', {}).get('frequency_hz')) or 10
    while not self.stop_process_signal.value:
      cycle_time = time.time()
      x, y, z = accel.GetGForces()
      point = gps_pb2.Point()
      point.accelerometer_x = x
      point.accelerometer_y = y
      point.accelerometer_z = z
      pitch, roll = accel.CalcPitchAndRoll(x, y, z)
      point.pitch = pitch
      point.roll = roll
      self.AddPointToQueue(point)
      time.sleep(sensor.SleepBasedOnHertz(cycle_time, frequency_hz))



def main(unused_argv):
  accel = Accelerometer()
  while True:
    x, y, z = accel.GetGForces()
    print('%.2f %.2f %.2f' % (x, y, z))
    print('Roll: %.2f, Pitch: %.2f' % accel.CalcPitchAndRoll(x, y, z))
    time.sleep(1)


if __name__ == '__main__':
  app.run(main)
